@@ -1,3 +1,4 @@
+# 2 "asmcomp/amd64/selection.ml"
 (**************************************************************************)
 (*                                                                        *)
 (*                                 OCaml                                  *)
@@ -30,38 +31,39 @@ type addressing_expr =
   | Ascaledadd of expression * expression * int
 
 let rec select_addr exp =
+  let default = (Alinear exp, 0) in
   match exp with
     Cconst_symbol (s, _) when not !Clflags.dlcode ->
       (Asymbol s, 0)
-  | Cop((Caddi | Caddv | Cadda), [arg; Cconst_int (m, _)], _) ->
-      let (a, n) = select_addr arg in (a, n + m)
-  | Cop(Csubi, [arg; Cconst_int (m, _)], _) ->
-      let (a, n) = select_addr arg in (a, n - m)
+  | Cop((Caddi | Caddv | Cadda), [arg; Cconst_int (m, _)], _)
   | Cop((Caddi | Caddv | Cadda), [Cconst_int (m, _); arg], _) ->
-      let (a, n) = select_addr arg in (a, n + m)
+      let (a, n) = select_addr arg in
+      if Misc.no_overflow_add n m then (a, n + m) else default
+  | Cop(Csubi, [arg; Cconst_int (m, _)], _) ->
+      let (a, n) = select_addr arg in
+      if Misc.no_overflow_sub n m then (a, n - m) else default
   | Cop(Clsl, [arg; Cconst_int((1|2|3 as shift), _)], _) ->
       begin match select_addr arg with
-        (Alinear e, n) -> (Ascale(e, 1 lsl shift), n lsl shift)
-      | _ -> (Alinear exp, 0)
-      end
-  | Cop(Cmuli, [arg; Cconst_int((2|4|8 as mult), _)], _) ->
-      begin match select_addr arg with
-        (Alinear e, n) -> (Ascale(e, mult), n * mult)
-      | _ -> (Alinear exp, 0)
+      | (Alinear e, n) when Misc.no_overflow_lsl n shift ->
+          (Ascale(e, 1 lsl shift), n lsl shift)
+      | _ -> default
       end
+  | Cop(Cmuli, [arg; Cconst_int((2|4|8 as mult), _)], _)
   | Cop(Cmuli, [Cconst_int((2|4|8 as mult), _); arg], _) ->
       begin match select_addr arg with
-        (Alinear e, n) -> (Ascale(e, mult), n * mult)
-      | _ -> (Alinear exp, 0)
+      | (Alinear e, n) when Misc.no_overflow_mul n mult ->
+          (Ascale(e, mult), n * mult)
+      | _ -> default
       end
   | Cop((Caddi | Caddv | Cadda), [arg1; arg2], _) ->
       begin match (select_addr arg1, select_addr arg2) with
-          ((Alinear e1, n1), (Alinear e2, n2)) ->
+          ((Alinear e1, n1), (Alinear e2, n2))
+          when Misc.no_overflow_add n1 n2 ->
               (Aadd(e1, e2), n1 + n2)
-        | ((Alinear e1, n1), (Ascale(e2, scale), n2)) ->
+        | ((Alinear e1, n1), (Ascale(e2, scale), n2))
+        | ((Ascale(e2, scale), n2), (Alinear e1, n1))
+          when Misc.no_overflow_add n1 n2 ->
               (Ascaledadd(e1, e2, scale), n1 + n2)
-        | ((Ascale(e1, scale), n1), (Alinear e2, n2)) ->
-              (Ascaledadd(e2, e1, scale), n1 + n2)
         | (_, (Ascale(e2, scale), n2)) ->
               (Ascaledadd(arg1, e2, scale), n2)
         | ((Ascale(e1, scale), n1), _) ->
@@ -69,8 +71,7 @@ let rec select_addr exp =
         | _ ->
               (Aadd(arg1, arg2), 0)
       end
-  | arg ->
-      (Alinear arg, 0)
+  | _ -> default
 
 (* Special constraints on operand and result registers *)
 
@@ -112,6 +113,18 @@ let pseudoregs_for_operation op arg res =
       ([| rax; rcx |], [| rax |])
   | Iintop(Imod) ->
       ([| rax; rcx |], [| rdx |])
+  | Icompf cond ->
+      (* We need to temporarily store the result of the comparison in a
+         float register, but we don't want to clobber any of the inputs
+         if they would still be live after this operation -- so we
+         add a fresh register as both an input and output. We don't use
+         [destroyed_at_oper], because that forces us to choose a fixed
+         register, which makes it more likely an extra mov would be added
+         to transfer the argument to the fixed register. *)
+      let treg = Reg.create Float in
+      let _,is_swapped = float_cond_and_need_swap cond in
+      (if is_swapped then [| arg.(0); treg |] else [| treg; arg.(1) |])
+    , [| res.(0); treg |]
   (* Other instructions are regular *)
   | _ -> raise Use_default
 
@@ -121,17 +134,24 @@ let inline_ops =
   [ "sqrt"; "caml_bswap16_direct"; "caml_int32_direct_bswap";
     "caml_int64_direct_bswap"; "caml_nativeint_direct_bswap" ]
 
+let is_immediate n = n <= 0x7FFF_FFFF && n >= -0x8000_0000
+
+let is_immediate_natint n = n <= 0x7FFF_FFFFn && n >= -0x8000_0000n
+
 (* The selector class *)
 
 class selector = object (self)
 
-inherit Spacetime_profiling.instruction_selection as super
+inherit Selectgen.selector_generic as super
 
-method is_immediate n = n <= 0x7FFF_FFFF && n >= (-1-0x7FFF_FFFF)
-  (* -1-.... : hack so that this can be compiled on 32-bit
-     (cf 'make check_all_arches') *)
+method! is_immediate op n =
+  match op with
+  | Iadd | Isub | Imul | Iand | Ior | Ixor | Icomp _ | Icheckbound ->
+      is_immediate n
+  | _ ->
+      super#is_immediate op n
 
-method is_immediate_natint n = n <= 0x7FFFFFFFn && n >= -0x80000000n
+method is_immediate_test _cmp n = is_immediate n
 
 method! is_simple_expr e =
   match e with
@@ -153,7 +173,7 @@ method! effects_of e =
 method select_addressing _chunk exp =
   let (a, d) = select_addr exp in
   (* PR#4625: displacement must be a signed 32-bit immediate *)
-  if not (self # is_immediate d)
+  if not (is_immediate d)
   then (Iindexed 0, exp)
   else match a with
     | Asymbol s ->
@@ -169,16 +189,9 @@ method select_addressing _chunk exp =
 
 method! select_store is_assign addr exp =
   match exp with
-    Cconst_int (n, _dbg) when self#is_immediate n ->
+    Cconst_int (n, _dbg) when is_immediate n ->
       (Ispecific(Istore_int(Nativeint.of_int n, addr, is_assign)), Ctuple [])
-  | (Cconst_natint (n, _dbg)) when self#is_immediate_natint n ->
-      (Ispecific(Istore_int(n, addr, is_assign)), Ctuple [])
-  | (Cblockheader(n, _dbg))
-      when self#is_immediate_natint n && not Config.spacetime ->
-      (Ispecific(Istore_int(n, addr, is_assign)), Ctuple [])
-  | Cconst_pointer (n, _dbg) when self#is_immediate n ->
-      (Ispecific(Istore_int(Nativeint.of_int n, addr, is_assign)), Ctuple [])
-  | Cconst_natpointer (n, _dbg) when self#is_immediate_natint n ->
+  | (Cconst_natint (n, _dbg)) when is_immediate_natint n ->
       (Ispecific(Istore_int(n, addr, is_assign)), Ctuple [])
   | _ ->
       super#select_store is_assign addr exp
@@ -201,9 +214,9 @@ method! select_operation op args dbg =
       self#select_floatarith true Imulf Ifloatmul args
   | Cdivf ->
       self#select_floatarith false Idivf Ifloatdiv args
-  | Cextcall("sqrt", _, false, _) ->
+  | Cextcall("sqrt", _, _, false) ->
      begin match args with
-       [Cop(Cload ((Double|Double_u as chunk), _), [loc], _dbg)] ->
+       [Cop(Cload {memory_chunk=(Double as chunk)}, [loc], _dbg)] ->
          let (addr, arg) = self#select_addressing chunk loc in
          (Ispecific(Ifloatsqrtf addr), [arg])
      | [arg] ->
@@ -215,7 +228,7 @@ method! select_operation op args dbg =
   | Cstore ((Word_int|Word_val as chunk), _init) ->
       begin match args with
         [loc; Cop(Caddi, [Cop(Cload _, [loc'], _); Cconst_int (n, _dbg)], _)]
-        when loc = loc' && self#is_immediate n ->
+        when loc = loc' && is_immediate n ->
           let (addr, arg) = self#select_addressing chunk loc in
           (Ispecific(Ioffset_loc(n, addr)), [arg])
       | _ ->
@@ -228,12 +241,9 @@ method! select_operation op args dbg =
   | Cextcall("caml_int64_direct_bswap", _, _, _)
   | Cextcall("caml_nativeint_direct_bswap", _, _, _) ->
       (Ispecific (Ibswap 64), args)
-  (* AMD64 does not support immediate operands for multiply high signed *)
-  | Cmulhi ->
-      (Iintop Imulh, args)
+  (* Recognize sign extension *)
   | Casr ->
       begin match args with
-        (* Recognize sign extension *)
         [Cop(Clsl, [k; Cconst_int (32, _)], _); Cconst_int (32, _)] ->
           (Ispecific Isextend32, [k])
         | _ -> super#select_operation op args dbg
@@ -254,11 +264,11 @@ method! select_operation op args dbg =
 
 method select_floatarith commutative regular_op mem_op args =
   match args with
-    [arg1; Cop(Cload ((Double|Double_u as chunk), _), [loc2], _)] ->
+    [arg1; Cop(Cload {memory_chunk=(Double as chunk)}, [loc2], _)] ->
       let (addr, arg2) = self#select_addressing chunk loc2 in
       (Ispecific(Ifloatarithmem(mem_op, addr)),
                  [arg1; arg2])
-  | [Cop(Cload ((Double|Double_u as chunk), _), [loc1], _); arg2]
+  | [Cop(Cload {memory_chunk=(Double as chunk)}, [loc1], _); arg2]
         when commutative ->
       let (addr, arg1) = self#select_addressing chunk loc1 in
       (Ispecific(Ifloatarithmem(mem_op, addr)),
@@ -268,9 +278,6 @@ method select_floatarith commutative regular_op mem_op args =
   | _ ->
       assert false
 
-method! mark_c_tailcall =
-  contains_calls := true
-
 (* Deal with register constraints *)
 
 method! insert_op_debug env op dbg rs rd =
@@ -285,4 +292,5 @@ method! insert_op_debug env op dbg rs rd =
 
 end
 
-let fundecl f = (new selector)#emit_fundecl f
+let fundecl ~future_funcnames f =
+  (new selector)#emit_fundecl ~future_funcnames f
