@@ -13,29 +13,45 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* From lambda to assembly code *)
+(** From Lambda to assembly code *)
 
-val compile_implementation_flambda :
-    ?toplevel:(string -> bool) ->
-    string ->
-    required_globals:Ident.Set.t ->
-    backend:(module Backend_intf.S) ->
-    ppf_dump:Format.formatter -> Flambda.program -> unit
+(** The type of converters from Lambda to Clambda. *)
+type middle_end =
+     backend:(module Backend_intf.S)
+  -> prefixname:string
+  -> ppf_dump:Format.formatter
+  -> Lambda.program
+  -> Clambda.with_constants
 
-val compile_implementation_clambda :
-    ?toplevel:(string -> bool) ->
-    string ->
-    ppf_dump:Format.formatter -> Lambda.program -> unit
+(** Compile an implementation from Lambda using the given middle end. *)
+val compile_implementation
+   : ?toplevel:(string -> bool)
+  -> backend:(module Backend_intf.S)
+  -> prefixname:string
+  -> middle_end:middle_end
+  -> ppf_dump:Format.formatter
+  -> Lambda.program
+  -> unit
+
+val compile_implementation_linear :
+    Unit_info.t -> unit
 
 val compile_phrase :
     ppf_dump:Format.formatter -> Cmm.phrase -> unit
 
-type error = Assembler_error of string
+type error =
+  | Assembler_error of string
+  | Mismatched_for_pack of string option
+  | Asm_generation of string * Emitaux.error
+
 exception Error of error
-val report_error: Format.formatter -> error -> unit
+val report_error: error Format_doc.format_printer
+val report_error_doc: error Format_doc.printer
 
-
-val compile_unit:
-  string(*prefixname*) ->
-  string(*asm file*) -> bool(*keep asm*) ->
-  string(*obj file*) -> (unit -> unit) -> unit
+val compile_unit
+   : output_prefix:string
+   -> asm_filename:string
+   -> keep_asm:bool
+   -> obj_filename:string
+   -> (unit -> unit)
+   -> unit
