@@ -18,15 +18,15 @@ open Mach
 open Linear
 
 module Make (T : Branch_relaxation_intf.S) = struct
-  let label_map code =
+  let label_map f =
     let map = Hashtbl.create 37 in
     let rec fill_map pc instr =
       match instr.desc with
       | Lend -> (pc, map)
       | Llabel lbl -> Hashtbl.add map lbl pc; fill_map pc instr.next
-      | op -> fill_map (pc + T.instr_size op) instr.next
+      | op -> fill_map (pc + T.instr_size f op) instr.next
     in
-    fill_map 0 code
+    fill_map 0 f.fun_body
 
   let branch_overflows map pc_branch lbl_dest max_branch_offset =
     let pc_dest = Hashtbl.find map lbl_dest in
@@ -51,8 +51,9 @@ module Make (T : Branch_relaxation_intf.S) = struct
       in
       match instr.desc with
       | Lop (Ialloc _)
-      | Lop (Iintop (Icheckbound _))
-      | Lop (Iintop_imm (Icheckbound _, _))
+      | Lop (Ipoll { return_label = None })
+      | Lop (Iintop (Icheckbound))
+      | Lop (Iintop_imm (Icheckbound, _))
       | Lop (Ispecific _) ->
         (* We assume that any branches eligible for relaxation generated
            by these instructions only branch forward.  We further assume
@@ -64,10 +65,15 @@ module Make (T : Branch_relaxation_intf.S) = struct
         opt_branch_overflows map pc lbl0 max_branch_offset
           || opt_branch_overflows map pc lbl1 max_branch_offset
           || opt_branch_overflows map pc lbl2 max_branch_offset
+      | Lop (Ipoll { return_label = Some lbl }) ->
+        (* A poll-and-branch instruction can branch to the label lbl,
+           but also to an out-of-line code block. *)
+        code_size + max_out_of_line_code_offset - pc >= max_branch_offset
+        || branch_overflows map pc lbl max_branch_offset
       | _ ->
         Misc.fatal_error "Unsupported instruction for branch relaxation"
 
-  let fixup_branches ~code_size ~max_out_of_line_code_offset map code =
+  let fixup_branches ~code_size ~max_out_of_line_code_offset map f =
     let expand_optbranch lbl n arg next =
       match lbl with
       | None -> next
@@ -83,23 +89,25 @@ module Make (T : Branch_relaxation_intf.S) = struct
           instr_overflows ~code_size ~max_out_of_line_code_offset instr map pc
         in
         if not overflows then
-          fixup did_fix (pc + T.instr_size instr.desc) instr.next
+          fixup did_fix (pc + T.instr_size f instr.desc) instr.next
         else
           match instr.desc with
-          | Lop (Ialloc { bytes = num_bytes; label_after_call_gc; dbginfo }) ->
-            instr.desc <- T.relax_allocation ~num_bytes
-                            ~dbginfo ~label_after_call_gc;
-            fixup true (pc + T.instr_size instr.desc) instr.next
-          | Lop (Iintop (Icheckbound { label_after_error; })) ->
-            instr.desc <- T.relax_intop_checkbound ~label_after_error;
-            fixup true (pc + T.instr_size instr.desc) instr.next
-          | Lop (Iintop_imm (Icheckbound { label_after_error; }, bound)) ->
+          | Lop (Ipoll { return_label }) ->
+            instr.desc <- T.relax_poll ~return_label;
+            fixup true (pc + T.instr_size f instr.desc) instr.next
+          | Lop (Ialloc { bytes = num_bytes; dbginfo }) ->
+            instr.desc <- T.relax_allocation ~num_bytes ~dbginfo;
+            fixup true (pc + T.instr_size f instr.desc) instr.next
+          | Lop (Iintop (Icheckbound)) ->
+            instr.desc <- T.relax_intop_checkbound ();
+            fixup true (pc + T.instr_size f instr.desc) instr.next
+          | Lop (Iintop_imm (Icheckbound, bound)) ->
             instr.desc
-              <- T.relax_intop_imm_checkbound ~bound ~label_after_error;
-            fixup true (pc + T.instr_size instr.desc) instr.next
+              <- T.relax_intop_imm_checkbound ~bound;
+            fixup true (pc + T.instr_size f instr.desc) instr.next
           | Lop (Ispecific specific) ->
             instr.desc <- T.relax_specific_op specific;
-            fixup true (pc + T.instr_size instr.desc) instr.next
+            fixup true (pc + T.instr_size f instr.desc) instr.next
           | Lcondbranch (test, lbl) ->
             let lbl2 = Cmm.new_label() in
             let cont =
@@ -108,7 +116,7 @@ module Make (T : Branch_relaxation_intf.S) = struct
             in
             instr.desc <- Lcondbranch (invert_test test, lbl2);
             instr.next <- cont;
-            fixup true (pc + T.instr_size instr.desc) instr.next
+            fixup true (pc + T.instr_size f instr.desc) instr.next
           | Lcondbranch3 (lbl0, lbl1, lbl2) ->
             let cont =
               expand_optbranch lbl0 0 instr.arg
@@ -124,20 +132,20 @@ module Make (T : Branch_relaxation_intf.S) = struct
                We can *never* get here. *)
             assert false
     in
-    fixup false 0 code
+    fixup false 0 f.fun_body
 
   (* Iterate branch expansion till all conditional branches are OK *)
 
-  let rec relax code ~max_out_of_line_code_offset =
+  let rec relax f ~max_out_of_line_code_offset =
     let min_of_max_branch_offsets =
       List.fold_left (fun min_of_max_branch_offsets branch ->
-          min min_of_max_branch_offsets
+          Int.min min_of_max_branch_offsets
             (T.Cond_branch.max_displacement branch))
         max_int T.Cond_branch.all
     in
-    let (code_size, map) = label_map code in
+    let (code_size, map) = label_map f in
     if code_size >= min_of_max_branch_offsets
-        && fixup_branches ~code_size ~max_out_of_line_code_offset map code
-    then relax code ~max_out_of_line_code_offset
+        && fixup_branches ~code_size ~max_out_of_line_code_offset map f
+    then relax f ~max_out_of_line_code_offset
     else ()
 end
