@@ -14,9 +14,23 @@
 
 [@@@ocaml.warning "-40"]
 
+module V = Backend_var
+module VP = Backend_var.With_provenance
 module Int = Numbers.Int
 
-(* Check a number of continuation-related invariants *)
+(* Check a number of invariants around continuation and variable uses *)
+
+type mutability = Mutable | Immutable
+
+let equal_mutability m1 m2 =
+  match m1, m2 with
+  | Mutable, Mutable | Immutable, Immutable -> true
+  | Mutable, Immutable | Immutable, Mutable -> false
+
+let mutability_to_string m =
+  match m with
+  | Mutable -> "mutable"
+  | Immutable -> "immutable"
 
 module Env : sig
   type t
@@ -27,10 +41,17 @@ module Env : sig
 
   val jump : t -> cont:int -> arg_num:int -> unit
 
+  val bind_var : t -> V.t -> mutability -> t
+
+  val bind_params : t -> (VP.t * _) list -> t
+
+  val use_var : t -> V.t -> mutability -> unit
+
   val report : Format.formatter -> bool
 end = struct
   type t = {
     bound_handlers : int Int.Map.t;
+    bound_variables : mutability V.Map.t;
   }
 
   type error =
@@ -38,6 +59,9 @@ end = struct
     | Multiple_handlers of { cont: int; }
     | Wrong_arguments_number of
         { cont: int; handler_args: int; jump_args: int; }
+    | Unbound_variable of { var : V.t; mut : mutability }
+    | Wrong_mutability of
+        { var : V.t; binding_mut : mutability; use_mut : mutability }
 
   module Error = struct
     type t = error
@@ -74,13 +98,14 @@ end = struct
     state.errors <- ErrorSet.empty;
     {
       bound_handlers = Int.Map.empty;
+      bound_variables = V.Map.empty;
     }
 
   let handler t ~cont ~arg_num =
     if Int.Set.mem cont state.all_handlers then multiple_handler cont;
     state.all_handlers <- Int.Set.add cont state.all_handlers;
     let bound_handlers = Int.Map.add cont arg_num t.bound_handlers in
-    { bound_handlers; }
+    { t with bound_handlers; }
 
   let jump t ~cont ~arg_num =
     match Int.Map.find cont t.bound_handlers with
@@ -89,6 +114,27 @@ end = struct
         wrong_arguments cont handler_args arg_num
     | exception Not_found -> unbound_handler cont
 
+  let bind_var t var mut =
+    let bound_variables = V.Map.add var mut t.bound_variables in
+    { t with bound_variables }
+
+  let bind_params t params =
+    let bound_variables =
+        List.fold_left (fun bound_vars (var, _) ->
+            V.Map.add (VP.var var) Immutable bound_vars)
+        t.bound_variables params
+    in
+    { t with bound_variables }
+
+  let use_var t var use_mut =
+    match V.Map.find_opt var t.bound_variables with
+    | Some binding_mut ->
+      if equal_mutability use_mut binding_mut
+      then ()
+      else record_error (Wrong_mutability { var; binding_mut; use_mut })
+    | None ->
+      record_error (Unbound_variable { var; mut = use_mut })
+
   let print_error ppf error =
     match error with
     | Unbound_handler { cont } ->
@@ -110,6 +156,16 @@ end = struct
         cont
         handler_args
         jump_args
+    | Unbound_variable { var; mut } ->
+      Format.fprintf ppf
+        "Variable %a (%s) was unbound or used outside the scope of its binder"
+        V.print var (mutability_to_string mut)
+    | Wrong_mutability { var; binding_mut; use_mut } ->
+      Format.fprintf ppf
+        "Variable %a was bound as %s but used as %s"
+        V.print var
+        (mutability_to_string binding_mut)
+        (mutability_to_string use_mut)
 
   let print_error_newline ppf error =
     Format.fprintf ppf "%a@." print_error error
@@ -125,15 +181,22 @@ end
 let rec check env (expr : Cmm.expression) =
   match expr with
   | Cconst_int _ | Cconst_natint _ | Cconst_float _ | Cconst_symbol _
-  | Cvar _ | Creturn_addr ->
+  | Creturn_addr ->
     ()
-  | Clet (_, expr, body)
-  | Clet_mut (_, _, expr, body) ->
+  | Cvar id ->
+    Env.use_var env id Immutable
+  | Cvar_mut id ->
+    Env.use_var env id Mutable
+  | Clet (id, expr, body) ->
     check env expr;
-    check env body
+    check (Env.bind_var env (VP.var id) Immutable) body
+  | Clet_mut (id, _, expr, body) ->
+    check env expr;
+    check (Env.bind_var env (VP.var id) Mutable) body
   | Cphantom_let (_, _, expr) ->
     check env expr
-  | Cassign (_, expr) ->
+  | Cassign (id, expr) ->
+    Env.use_var env id Mutable;
     check env expr
   | Ctuple exprs ->
     List.iter (check env) exprs
@@ -163,18 +226,21 @@ let rec check env (expr : Cmm.expression) =
       | Recursive -> env_extended
       | Nonrecursive -> env
     in
-    List.iter (fun (_, _, handler, _) -> check env_handler handler) handlers
+    List.iter (fun (_, args, handler, _) ->
+        let env_handler = Env.bind_params env_handler args in
+        check env_handler handler)
+      handlers
   | Cexit (cont, args) ->
     Env.jump env ~cont ~arg_num:(List.length args)
-  | Ctrywith (body, _, handler, _) ->
+  | Ctrywith (body, id, handler, _) ->
     (* Jumping from inside a trywith body to outside isn't very nice,
        but it's handled correctly by Linearize, as it happens
        when compiling match ... with exception ..., for instance, so it is
        not reported as an error. *)
     check env body;
-    check env handler
+    check (Env.bind_var env (VP.var id) Immutable) handler
 
 let run ppf (fundecl : Cmm.fundecl) =
-  let env = Env.init () in
+  let env = Env.bind_params (Env.init ()) fundecl.fun_args in
   check env fundecl.fun_body;
   Env.report ppf
