@@ -19,5 +19,7 @@ open Lambda
 open Instruct
 
 val compile_implementation: string -> lambda -> instruction list
-val compile_phrase: lambda -> instruction list * instruction list
-val reset: unit -> unit
+val compile_phrase: lambda -> instruction list * bool
+
+val merge_events:
+  Instruct.debug_event -> Instruct.debug_event -> Instruct.debug_event
