@@ -19,7 +19,7 @@ type dll_handle
 type dll_address
 type dll_mode = For_checking | For_execution
 
-external dll_open: dll_mode -> string -> dll_handle = "caml_dynlink_open_lib"
+external dll_open: string -> dll_handle = "caml_dynlink_open_lib"
 external dll_close: dll_handle -> unit = "caml_dynlink_close_lib"
 external dll_sym: dll_handle -> string -> dll_address
                 = "caml_dynlink_lookup_symbol"
@@ -31,11 +31,16 @@ external get_current_dlls: unit -> dll_handle array
 (* Current search path for DLLs *)
 let search_path = ref ([] : string list)
 
-(* DLLs currently opened *)
-let opened_dlls = ref ([] : dll_handle list)
+type opened_dll =
+  | Checking of Binutils.t
+  | Execution of dll_handle
 
-(* File names for those DLLs *)
-let names_of_opened_dlls = ref ([] : string list)
+let dll_close = function
+  | Checking _ -> ()
+  | Execution dll -> dll_close dll
+
+(* DLLs currently opened *)
+let opened_dlls = ref ([] : (string * opened_dll) list)
 
 (* Add the given directories to the search path for DLLs. *)
 let add_path dirs =
@@ -66,14 +71,26 @@ let open_dll mode name =
         Filename.concat Filename.current_dir_name fullname
       else fullname
     with Not_found -> name in
-  if not (List.mem fullname !names_of_opened_dlls) then begin
-    try
-      let dll = dll_open mode fullname in
-      names_of_opened_dlls := fullname :: !names_of_opened_dlls;
-      opened_dlls := dll :: !opened_dlls
-    with Failure msg ->
-      failwith (fullname ^ ": " ^ msg)
-  end
+  match List.assoc_opt fullname !opened_dlls, mode with
+  | Some (Execution _), (For_execution | For_checking) -> ()
+  | Some (Checking _), For_checking -> ()
+  | None, For_checking ->
+      begin match Binutils.read fullname with
+      | Ok t -> opened_dlls := (fullname, Checking t) :: !opened_dlls
+      | Error err ->
+          failwith (fullname ^ ": " ^ Binutils.error_to_string err)
+      end
+  | (None | Some (Checking _) as current), For_execution ->
+      begin match dll_open fullname with
+      | dll ->
+          let opened = match current with
+            | None -> List.remove_assoc fullname !opened_dlls
+            | Some _ -> !opened_dlls
+          in
+          opened_dlls := (fullname, Execution dll) :: opened
+      | exception Failure msg ->
+          failwith (fullname ^ ": " ^ msg)
+      end
 
 let open_dlls mode names =
   List.iter (open_dll mode) names
@@ -81,23 +98,31 @@ let open_dlls mode names =
 (* Close all DLLs *)
 
 let close_all_dlls () =
-  List.iter dll_close !opened_dlls;
+  List.iter (fun (_, dll) -> dll_close dll) !opened_dlls;
   opened_dlls := [];
-  names_of_opened_dlls := []
 
-(* Find a primitive in the currently opened DLLs.
-   Raise [Not_found] if not found. *)
+(* Find a primitive in the currently opened DLLs. *)
+
+type primitive_address =
+  | Prim_loaded of dll_address
+  | Prim_exists
 
 let find_primitive prim_name =
   let rec find seen = function
     [] ->
-      raise Not_found
-  | dll :: rem ->
+      None
+  | (_,Execution dll) as curr :: rem ->
       let addr = dll_sym dll prim_name in
-      if addr == Obj.magic () then find (dll :: seen) rem else begin
-        if seen <> [] then opened_dlls := dll :: List.rev_append seen rem;
-        addr
-      end in
+      if addr == Obj.magic () then find (curr :: seen) rem else begin
+        if seen <> [] then opened_dlls := curr :: List.rev_append seen rem;
+        Some (Prim_loaded addr)
+      end
+  | (_,Checking t) as curr :: rem ->
+      if Binutils.defines_symbol t prim_name then
+        Some Prim_exists
+      else
+        find (curr :: seen) rem
+  in
   find [] !opened_dlls
 
 (* If linking in core (dynlink or toplevel), synchronize the VM
@@ -139,9 +164,6 @@ let ld_library_path_contents () =
   | s ->
       Misc.split_path_contents s
 
-let split_dll_path path =
-  Misc.split_path_contents ~sep:'\000' path
-
 (* Initialization for separate compilation *)
 
 let init_compile nostdlib =
@@ -151,17 +173,14 @@ let init_compile nostdlib =
 
 (* Initialization for linking in core (dynlink or toplevel) *)
 
-let init_toplevel dllpath =
-  search_path :=
-    ld_library_path_contents() @
-    split_dll_path dllpath @
-    ld_conf_contents();
-  opened_dlls := Array.to_list (get_current_dlls());
-  names_of_opened_dlls := [];
+let init_toplevel dllpaths =
+  search_path := dllpaths;
+  opened_dlls :=
+    List.map (fun dll -> "", Execution dll)
+      (Array.to_list (get_current_dlls()));
   linking_in_core := true
 
 let reset () =
   search_path := [];
   opened_dlls :=[];
-  names_of_opened_dlls := [];
   linking_in_core := false
