@@ -38,7 +38,7 @@ let marshal_to_channel_with_possibly_32bit_compat ~filename ~kind outchan obj =
 
 
 let report_error ppf (file, kind) =
-  Format.fprintf ppf "Generated %s %S cannot be used on a 32-bit platform"
+  Format_doc.fprintf ppf "Generated %s %S cannot be used on a 32-bit platform"
                      kind file
 let () =
   Location.register_error_of_exn
@@ -50,22 +50,30 @@ let () =
     )
 
 (* Buffering of bytecode *)
+let create_bigarray = Bigarray.Array1.create Bigarray.Char Bigarray.c_layout
 
-let out_buffer = ref(LongString.create 1024)
+let copy_bigarray src dst size =
+  Bigarray.Array1.(blit (sub src 0 size) (sub dst 0 size))
+
+let out_buffer = ref(create_bigarray 0)
 and out_position = ref 0
 
+let extend_buffer needed =
+  let size = Bigarray.Array1.dim !out_buffer in
+  let new_size = ref(max size 16) (* we need new_size > 0 *) in
+  while needed >= !new_size do new_size := 2 * !new_size done;
+  let new_buffer = create_bigarray !new_size in
+  copy_bigarray !out_buffer new_buffer size;
+  out_buffer := new_buffer
+
 let out_word b1 b2 b3 b4 =
   let p = !out_position in
-  if p >= LongString.length !out_buffer then begin
-    let len = LongString.length !out_buffer in
-    let new_buffer = LongString.create (2 * len) in
-    LongString.blit !out_buffer 0 new_buffer 0 len;
-    out_buffer := new_buffer
-  end;
-  LongString.set !out_buffer p (Char.unsafe_chr b1);
-  LongString.set !out_buffer (p+1) (Char.unsafe_chr b2);
-  LongString.set !out_buffer (p+2) (Char.unsafe_chr b3);
-  LongString.set !out_buffer (p+3) (Char.unsafe_chr b4);
+  let open Bigarray.Array1 in
+  if p+3 >= dim !out_buffer then extend_buffer (p+3);
+  set !out_buffer p (Char.unsafe_chr b1);
+  set !out_buffer (p+1) (Char.unsafe_chr b2);
+  set !out_buffer (p+2) (Char.unsafe_chr b3);
+  set !out_buffer (p+3) (Char.unsafe_chr b4);
   out_position := p + 4
 
 let out opcode =
@@ -77,7 +85,6 @@ exception AsInt
 let const_as_int = function
   | Const_base(Const_int i) -> i
   | Const_base(Const_char c) -> Char.code c
-  | Const_pointer i -> i
   | _ -> raise AsInt
 
 let is_immed i = immed_min <= i && i <= immed_max
@@ -107,7 +114,8 @@ type label_definition =
 let label_table  = ref ([| |] : label_definition array)
 
 let extend_label_table needed =
-  let new_size = ref(Array.length !label_table) in
+  let size = Array.length !label_table in
+  let new_size = ref(max size 16) (* we need new_size > 0 *) in
   while needed >= !new_size do new_size := 2 * !new_size done;
   let new_table = Array.make !new_size (Label_undefined []) in
   Array.blit !label_table 0 new_table 0 (Array.length !label_table);
@@ -115,10 +123,11 @@ let extend_label_table needed =
 
 let backpatch (pos, orig) =
   let displ = (!out_position - orig) asr 2 in
-  LongString.set !out_buffer pos (Char.unsafe_chr displ);
-  LongString.set !out_buffer (pos+1) (Char.unsafe_chr (displ asr 8));
-  LongString.set !out_buffer (pos+2) (Char.unsafe_chr (displ asr 16));
-  LongString.set !out_buffer (pos+3) (Char.unsafe_chr (displ asr 24))
+  let open Bigarray.Array1 in
+  set !out_buffer pos (Char.unsafe_chr displ);
+  set !out_buffer (pos+1) (Char.unsafe_chr (displ asr 8));
+  set !out_buffer (pos+2) (Char.unsafe_chr (displ asr 16));
+  set !out_buffer (pos+3) (Char.unsafe_chr (displ asr 24))
 
 let define_label lbl =
   if lbl >= Array.length !label_table then extend_label_table lbl;
@@ -149,13 +158,24 @@ let enter info =
   reloc_info := (info, !out_position) :: !reloc_info
 
 let slot_for_literal sc =
-  enter (Reloc_literal sc);
+  enter (Reloc_literal (Symtable.transl_const sc));
   out_int 0
 and slot_for_getglobal id =
-  enter (Reloc_getglobal id);
+  let name = Ident.name id in
+  let reloc_info =
+    if Ident.is_predef id then (Reloc_getpredef (Predef_exn name))
+    else if Ident.global id then (Reloc_getcompunit (Compunit name))
+    else assert false
+  in
+  enter reloc_info;
   out_int 0
 and slot_for_setglobal id =
-  enter (Reloc_setglobal id);
+  let name = Ident.name id in
+  let reloc_info =
+    if Ident.persistent id then (Reloc_setcompunit (Compunit name))
+    else assert false
+  in
+  enter reloc_info;
   out_int 0
 and slot_for_c_prim name =
   enter (Reloc_primitive name);
@@ -179,12 +199,18 @@ let record_event ev =
 
 (* Initialization *)
 
-let init () =
+let clear() =
   out_position := 0;
-  label_table := Array.make 16 (Label_undefined []);
+  label_table := [||];
   reloc_info := [];
   debug_dirs := String.Set.empty;
-  events := []
+  events := [];
+  out_buffer := create_bigarray 0
+
+let init () =
+  clear ();
+  label_table := Array.make 16 (Label_undefined []);
+  out_buffer := create_bigarray 1024
 
 (* Emission of one instruction *)
 
@@ -227,8 +253,8 @@ let emit_instr = function
       let org = !out_position in
       List.iter (out_label_with_orig org) lbls
   | Koffsetclosure ofs ->
-      if ofs = -2 || ofs = 0 || ofs = 2
-      then out (opOFFSETCLOSURE0 + ofs / 2)
+      if ofs = -3 || ofs = 0 || ofs = 3
+      then out (opOFFSETCLOSURE0 + ofs / 3)
       else (out opOFFSETCLOSURE; out_int ofs)
   | Kgetglobal q -> out opGETGLOBAL; slot_for_getglobal q
   | Ksetglobal q -> out opSETGLOBAL; slot_for_setglobal q
@@ -240,10 +266,6 @@ let emit_instr = function
           else (out opCONSTINT; out_int i)
       | Const_base(Const_char c) ->
           out opCONSTINT; out_int (Char.code c)
-      | Const_pointer i ->
-          if i >= 0 && i <= 3
-          then out (opCONST0 + i)
-          else (out opCONSTINT; out_int i)
       | Const_block(t, []) ->
           if t = 0 then out opATOM0 else (out opATOM; out_int t)
       | _ ->
@@ -305,6 +327,10 @@ let emit_instr = function
   | Kgetpubmet tag -> out opGETPUBMET; out_int tag; out_int 0
   | Kgetdynmet -> out opGETDYNMET
   | Kevent ev -> record_event ev
+  | Kperform -> out opPERFORM
+  | Kresume -> out opRESUME
+  | Kresumeterm n -> out opRESUMETERM; out_int n
+  | Kreperformterm n -> out opREPERFORMTERM; out_int n
   | Kstop -> out opSTOP
 
 (* Emission of a list of instructions. Include some peephole optimization. *)
@@ -356,8 +382,8 @@ let rec emit = function
       else (out opPUSHENVACC; out_int n);
       emit c
   | Kpush :: Koffsetclosure ofs :: c ->
-      if ofs = -2 || ofs = 0 || ofs = 2
-      then out(opPUSHOFFSETCLOSURE0 + ofs / 2)
+      if ofs = -3 || ofs = 0 || ofs = 3
+      then out(opPUSHOFFSETCLOSURE0 + ofs / 3)
       else (out opPUSHOFFSETCLOSURE; out_int ofs);
       emit c
   | Kpush :: Kgetglobal id :: Kgetfield n :: c ->
@@ -372,10 +398,6 @@ let rec emit = function
           else (out opPUSHCONSTINT; out_int i)
       | Const_base(Const_char c) ->
           out opPUSHCONSTINT; out_int(Char.code c)
-      | Const_pointer i ->
-          if i >= 0 && i <= 3
-          then out (opPUSHCONST0 + i)
-          else (out opPUSHCONSTINT; out_int i)
       | Const_block(t, []) ->
           if t = 0 then out opPUSHATOM0 else (out opPUSHATOM; out_int t)
       | _ ->
@@ -397,72 +419,74 @@ let rec emit = function
 
 (* Emission to a file *)
 
-let to_file outchan unit_name objfile ~required_globals code =
+let to_file outchan artifact_info ~required_globals code =
   init();
+  Fun.protect ~finally:clear (fun () ->
   output_string outchan cmo_magic_number;
   let pos_depl = pos_out outchan in
   output_binary_int outchan 0;
   let pos_code = pos_out outchan in
   emit code;
-  LongString.output outchan !out_buffer 0 !out_position;
+  Out_channel.output_bigarray outchan !out_buffer 0 !out_position;
   let (pos_debug, size_debug) =
     if !Clflags.debug then begin
+      let filename = Unit_info.Artifact.filename artifact_info in
       debug_dirs := String.Set.add
-        (Filename.dirname (Location.absolute_path objfile))
+          (Filename.dirname (Location.absolute_path filename))
         !debug_dirs;
       let p = pos_out outchan in
-      output_value outchan !events;
-      output_value outchan (String.Set.elements !debug_dirs);
+      Compression.output_value outchan !events;
+      Compression.output_value outchan (String.Set.elements !debug_dirs);
       (p, pos_out outchan - p)
     end else
       (0, 0) in
   let compunit =
-    { cu_name = unit_name;
+    { cu_name = Cmo_format.Compunit (Unit_info.Artifact.modname artifact_info);
       cu_pos = pos_code;
       cu_codesize = !out_position;
       cu_reloc = List.rev !reloc_info;
       cu_imports = Env.imports();
       cu_primitives = List.map Primitive.byte_name
                                !Translmod.primitive_declarations;
-      cu_required_globals = Ident.Set.elements required_globals;
+      cu_required_compunits = List.map (fun id -> Compunit (Ident.name id))
+        (Ident.Set.elements required_globals);
       cu_force_link = !Clflags.link_everything;
       cu_debug = pos_debug;
       cu_debugsize = size_debug } in
-  init();                               (* Free out_buffer and reloc_info *)
-  Btype.cleanup_abbrev ();              (* Remove any cached abbreviation
-                                           expansion before saving *)
   let pos_compunit = pos_out outchan in
-  marshal_to_channel_with_possibly_32bit_compat
-    ~filename:objfile ~kind:"bytecode unit"
-    outchan compunit;
+  let () =
+    (* Remove any cached abbreviation expansion before marshaling.
+       See doc-comment for [Types.abbrev_memo] *)
+    Btype.cleanup_abbrev ();
+    marshal_to_channel_with_possibly_32bit_compat
+      ~filename:(Unit_info.Artifact.filename artifact_info)
+      ~kind:"bytecode unit"
+      outchan compunit
+  in
   seek_out outchan pos_depl;
-  output_binary_int outchan pos_compunit
+  output_binary_int outchan pos_compunit)
 
 (* Emission to a memory block *)
 
-let to_memory init_code fun_code =
+let to_memory instrs =
   init();
-  emit init_code;
-  emit fun_code;
-  let code = LongString.create !out_position in
-  LongString.blit !out_buffer 0 code 0 !out_position;
+  Fun.protect ~finally:clear (fun () ->
+  emit instrs;
+  let code = create_bigarray !out_position in
+  copy_bigarray !out_buffer code !out_position;
   let reloc = List.rev !reloc_info in
   let events = !events in
-  init();
-  (code, reloc, events)
+  (code, reloc, events))
 
 (* Emission to a file for a packed library *)
 
 let to_packed_file outchan code =
-  init();
+  init ();
+  Fun.protect ~finally:clear (fun () ->
   emit code;
-  LongString.output outchan !out_buffer 0 !out_position;
-  let reloc = !reloc_info in
-  init();
-  reloc
-
-let reset () =
-  out_buffer := LongString.create 1024;
-  out_position := 0;
-  label_table := [| |];
-  reloc_info := []
+  Out_channel.output_bigarray outchan !out_buffer 0 !out_position;
+  let reloc = List.rev !reloc_info in
+  let events = !events in
+  let debug_dirs = !debug_dirs in
+  let size = !out_position in
+  (size, reloc, events, debug_dirs))
