@@ -427,7 +427,7 @@ let sort_files_by_dependencies files =
 
 (* Init Hashtbl with all defined modules *)
   let files = List.map (fun (file, file_kind, deps, pp_deps) ->
-    let modname = Unit_info.modname_from_source file in
+    let modname = Unit_info.lax_modname_from_source file in
     let key = (modname, file_kind) in
     let new_deps = ref [] in
     Hashtbl.add h key (file, new_deps);
@@ -518,15 +518,15 @@ let process_mli_map =
                          String.Map.empty Pparse.Signature
 
 let parse_map fname =
-  let old_transp = !Clflags.transparent_modules in
-  Clflags.transparent_modules := true;
+  let old_no_alias_deps = !Clflags.no_alias_deps in
+  Clflags.no_alias_deps := true;
   let (deps, m) =
     process_file fname ~def:(String.Set.empty, String.Map.empty)
       ~ml_file:process_ml_map
       ~mli_file:process_mli_map
   in
-  Clflags.transparent_modules := old_transp;
-  let modname = Unit_info.modname_from_source fname in
+  Clflags.no_alias_deps := old_no_alias_deps;
+  let modname = Unit_info.lax_modname_from_source fname in
   if String.Map.is_empty m then
     report_err (Failure (fname ^ " : empty map file or parse error"));
   let mm = Depend.make_node m in
@@ -578,7 +578,7 @@ let run_main argv =
         " Generate dependencies on all files";
       "-allow-approx", Arg.Set allow_approximation,
         " Fallback to a lexer-based approximation on unparsable files";
-      "-as-map", Arg.Set Clflags.transparent_modules,
+      "-as-map", Arg.Set Clflags.no_alias_deps,
         " Omit delayed dependencies for module aliases (-no-alias-deps -w -49)";
         (* "compiler uses -no-alias-deps, and no module is coerced"; *)
       "-debug-map", Arg.Set debug,
@@ -594,6 +594,9 @@ let run_main argv =
         "<f>  Process <f> as a .ml file";
       "-intf", Arg.String (add_dep_arg (fun f -> Src (f, Some MLI))),
         "<f>  Process <f> as a .mli file";
+      "-keywords", Arg.String (fun s -> Clflags.keyword_edition := Some s ),
+      "<version+list>  set keywords following the <version+list> spec \
+       (see ocamlc)";
       "-map", Arg.String (add_dep_arg (fun f -> Map f)),
         "<f>  Read <f> and propagate delayed dependencies to following files";
       "-ml-synonym", Arg.String(add_to_synonym_list ml_synonyms),
