@@ -13,6 +13,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
+(* see high-level comments in switch.mli *)
 
 type 'a shared = Shared of 'a | Single of 'a
 
@@ -22,8 +23,6 @@ type ('a, 'ctx) t_store =
    act_store : 'ctx -> 'a -> int ;
    act_store_shared : 'ctx -> 'a -> int ; }
 
-exception Not_simple
-
 module type Stored = sig
   type t
   type key
@@ -115,52 +114,120 @@ sig
   val ltint : primitive
   val geint : primitive
   val gtint : primitive
-  type act
-  type loc
 
-  val bind : act -> (act -> act) -> act
-  val make_const : int -> act
-  val make_offset : act -> int -> act
-  val make_prim : primitive -> act list -> act
-  val make_isout : act -> act -> act
-  val make_isin : act -> act -> act
-  val make_if : act -> act -> act -> act
-  val make_switch : loc -> act -> int array -> act array -> act
+  type loc
+  type arg
+  type test
+  type act
+
+  val bind : arg -> (arg -> act) -> act
+  val make_const : int -> arg
+  val make_offset : arg -> int -> arg
+  val make_prim : primitive -> arg list -> test
+  val make_isout : arg -> arg -> test
+  val make_isin : arg -> arg -> test
+  val make_is_nonzero : arg -> test
+  val arg_as_test : arg -> test
+
+  val make_if : test -> act -> act -> act
+  val make_switch : loc -> arg -> int array -> act array -> act
+
   val make_catch : act -> int * (act -> act)
   val make_exit : int -> act
 end
 
-(* The module will ``produce good code for the case statement'' *)
-(*
+(* The module will ``produce good code for the case statement''
+
   Adaptation of
-   R.L. Berstein
+   Robert L. Berstein
    ``Producing good code for the case statement''
    Software Practice and Experience, 15(10) (1985)
- and
-   D.L. Spuler
+  and
+   Sampath Kannan and Todd A. Proebsting
+   ``Correction to ``Producing good code for the case statement'' ''
+   Software Practice and Experience, 24(2) (1993)
+  and
+   David L. Spuler
     ``Two-Way Comparison Search Trees, a Generalisation of Binary Search Trees
       and Split Trees''
     ``Compiler Code Generation for Multiway Branch Statement as
       a Static Search Problem''
    Technical Reports, James Cook University
-*)
-(*
-  Main adaptation is considering interval tests
- (implemented as one addition + one unsigned test and branch)
-  which leads to exhaustive search for finding the optimal
-  test sequence in small cases and heuristics otherwise.
+
+ The article of Bernstein considers how to compile C-style switches:
+ arrays of actions indexed over non-negative integers with some "missing"
+ cases that are sent to a default action.
+
+ The strategy proposed, which is followed in our implementation below,
+ is as follows:
+ 1. Compute a "clustering" of the cases as a disjoint union of smaller intervals
+    with a high enough "density" (few default cases on the interval).
+ 2. Generate "dense switch" code for each cluster, typically using a jump table.
+ 3. Generate a sequence of tests for the whole switch, whose leaves
+    are the dense switches generated for each cluster.
+
+ Berstein believes that computing the optimal clustering
+ (smaller number of clusters) is NP-complete, and only proposes
+ a suboptimal heuristic method. Kannan and Proebsting remark that it
+ can be solved by a quadratic dynamic algorithm, which is also used in
+ our implementation.
+
+ The article of Spuler explains how to generate good test sequences
+ (optimal in worst-case number of tests) for a two-way tests instead
+ of three-way tests: traditional dichotomic search assumes that we
+ check at each step whether the key is (1) equal to the pivot, (2)
+ strictly less or (3) strictly more, but the test instructions in our
+ intermediate representations typically only let us test for (1)
+ lesser or equal or (2) strictly bigger (or: (1) strictly less, (2)
+ bigger or equal, which is symmetric.). Spuler proves that, even in
+ this two-way setting, dichotomic search generates optimal test
+ sequences.
+
+ The code below uses two additional ideas from Luc Maranget.
+
+ 1. The code to compute an optimal sequence of tests also makes use of
+    an interval check (is the input in the range [m; n]), which
+    (as remarked by Bernstein) can be implemented efficiently as
+    a subtraction and an unsigned comparison. We don't know of an
+    efficient algorithm to compute optimal test sequences using both
+    comparison and interval checks, so instead:
+    a. on large input intervals, we use the dichotomy
+    b. on medium-sized input intervals, we use the best of
+       the dichotomy and an interval check carving out
+       exactly the lowest and highest cases
+    c. on small input intervals, we use optimal exhaustive search.
+
+ 2. The works of Bernstein and Kannan-Proebsting compute clusters of
+    sufficient density, where density is defined naturally as the
+    proportion of non-default cases. Maranget instead computes density
+    as the height of the test sequence divided by interval size
+    (note that the number of non-default cases is an upperbound on the
+    test sequence height, as the length of the linear test sequence).
+    As a result, sub-intervals that can be efficiently decided by
+    tests get a lower density, so they are more likely to be merged
+    into the toplevel test sequence instead of generating a less
+    compact jump table.
 *)
 module Make (Arg : S) =
 struct
 
-  type 'a inter =
-    {cases : (int * int * int) array ;
-     actions : 'a array}
+  (* A representation of switches over intervals rather than discrete
+     values the [cases] array stores triples [(low, high, act)], where
+     [low] is the lowest input value of the interval, [high] is the
+     highest input value, and [act] is an index into the [actions]
+     array.
 
-  type 'a t_ctx =  {off : int ; arg : 'a}
+     (There can be substantially less actions than intervals if many
+     actions are shared.)
+  *)
+  type 'a inter = {
+    cases : (int * int * int) array ;
+    actions : 'a array
+  }
 
-  let cut = ref 8
-  and more_cut = ref 16
+  let small_size_limit = 8
+
+  and medium_size_limit = 16
 
 (*
 let pint chan i =
@@ -187,7 +254,17 @@ let prerr_inter i = Printf.fprintf stderr
   and get_low cases i =
     let r,_,_ = cases.(i) in
     r
+  and get_high cases i =
+    let _,r,_ = cases.(i) in
+    r
 
+  (* a "cost" as a number of tests in the worst case;
+     [n] is the total number of tests
+     [ni] is the number of interval tests
+
+     If two choices have the same total number of tests, we will prefer
+     the one with less interval tests as they cost slightly more.
+  *)
   type ctests = {
     mutable n : int ;
     mutable ni : int ;
@@ -228,6 +305,10 @@ let pta chan t =
     t1.n <- t1.n + t2.n ;
     t1.ni <- t1.ni + t2.ni ;
 
+  (* Represents tests in a test sequence
+     [Inter (low, high)] is an interval test
+     [Sep bound] is [fun x -> x < bound]
+     [No] is when no tests are necessary. *)
   type t_ret = Inter of int * int  | Sep of int | No
 
 (*
@@ -258,25 +339,19 @@ let pret chan = function
           for i = 0 to len1-2 do
             r.(i) <- c1.(i)
           done ;
-
           let l =
-            if len1-2 >= 0 then begin
+            if len1 < 2 then l1
+            else begin (* 0 <= len1 - 2 < len1 *)
               let _,h,_ = r.(len1-2) in
-              if h+1 < l1 then
-                h+1
-              else
-                l1
-            end else
-              l1
+              min (h + 1) l1
+            end
           and h =
-            if 1 < len2-1 then begin
+            if len2 < 2 then h2
+            else begin (* 0 <= 1 < len2 *)
               let l,_,_ = c2.(1) in
-              if h2+1 < l then
-                l-1
-              else
-                h2
-            end else
-              h2 in
+              max h2 (l - 1)
+            end
+          in
           r.(len1-1) <- (l,h,act1) ;
           for i=1 to len2-1  do
             r.(len1-1+i) <- c2.(i)
@@ -395,7 +470,8 @@ let rec pkey chan  = function
 
   let ok_inter = ref false
 
-  let rec opt_count top cases =
+  (* Compute a good test sequence. *)
+  let rec opt_count cases =
     let key = make_key cases in
     try
       Hashtbl.find t key
@@ -407,31 +483,35 @@ let rec pkey chan  = function
           | 0 -> assert false
           | _ when same_act cases -> No, ({n=0; ni=0},{n=0; ni=0})
           | _ ->
-              if lcases < !cut then
-                enum top cases
-              else if lcases < !more_cut then
+              if lcases < small_size_limit then
+                enum cases
+              else if lcases < medium_size_limit then
                 heuristic cases
               else
                 divide cases in
         Hashtbl.add t key r ;
         r
 
+  (* Large inputs: dichotomic sequence. *)
   and divide cases =
     let lcases = Array.length cases in
     let m = lcases/2 in
     let _,left,right = coupe cases m in
     let ci = {n=1 ; ni=0}
     and cm = {n=1 ; ni=0}
-    and _,(cml,cleft) = opt_count false left
-    and _,(cmr,cright) = opt_count false right in
+    and _,(cml,cleft) = opt_count left
+    and _,(cmr,cright) = opt_count right in
     add_test ci cleft ;
     add_test ci cright ;
+    (* To compute a worst-case cost, we add the more costly of the
+       left/right branches to the running total. *)
     if less_tests cml cmr then
       add_test cm cmr
     else
       add_test cm cml ;
     Sep m,(cm, ci)
 
+  (* Medium-size inputs: dichotomy or interval tests. *)
   and heuristic cases =
     let lcases = Array.length cases in
 
@@ -443,8 +523,8 @@ let rec pkey chan  = function
         and _,_,act1 = cases.(lcases-1) in
         if act0 = act1 then begin
           let low, high, inside, outside = coupe_inter 1 (lcases-2) cases in
-          let _,(cmi,cinside) = opt_count false inside
-          and _,(cmo,coutside) = opt_count false outside
+          let _,(cmi,cinside) = opt_count inside
+          and _,(cmo,coutside) = opt_count outside
           and cmij = {n=1 ; ni=(if low=high then 0 else 1)}
           and cij = {n=1 ; ni=(if low=high then 0 else 1)} in
           add_test cij cinside ;
@@ -463,8 +543,8 @@ let rec pkey chan  = function
     else
       inter,cinter
 
-
-  and enum top cases =
+  (* Small inputs: exhaustive search for optimal sequence. *)
+  and enum cases =
     let lcases = Array.length cases in
     let lim, with_sep =
       let best = ref (-1) and best_cost = ref (too_much,too_much) in
@@ -473,8 +553,8 @@ let rec pkey chan  = function
         let _,left,right = coupe cases i in
         let ci = {n=1 ; ni=0}
         and cm = {n=1 ; ni=0}
-        and _,(cml,cleft) = opt_count false left
-        and _,(cmr,cright) = opt_count false right in
+        and _,(cml,cleft) = opt_count left
+        and _,(cmr,cright) = opt_count right in
         add_test ci cleft ;
         add_test ci cright ;
         if less_tests cml cmr then
@@ -485,8 +565,6 @@ let rec pkey chan  = function
         if
           less2tests (cm,ci) !best_cost
         then begin
-          if top then
-            Printf.fprintf stderr "Get it: %d\n" i ;
           best := i ;
           best_cost := (cm,ci)
         end
@@ -500,8 +578,8 @@ let rec pkey chan  = function
         for i=1 to lcases-2 do
           let low, high, inside, outside = coupe_inter i i cases in
           if low=high then begin
-            let _,(cmi,cinside) = opt_count false inside
-            and _,(cmo,coutside) = opt_count false outside
+            let _,(cmi,cinside) = opt_count inside
+            and _,(cmo,coutside) = opt_count outside
             and cmij = {n=1 ; ni=0}
             and cij = {n=1 ; ni=0} in
             add_test cij cinside ;
@@ -524,8 +602,8 @@ let rec pkey chan  = function
         for i=1 to lcases-2 do
           for j=i to lcases-2 do
             let low, high, inside, outside = coupe_inter i j cases in
-            let _,(cmi,cinside) = opt_count false inside
-            and _,(cmo,coutside) = opt_count false outside
+            let _,(cmi,cinside) = opt_count inside
+            and _,(cmo,coutside) = opt_count outside
             and cmij = {n=1 ; ni=(if low=high then 0 else 1)}
             and cij = {n=1 ; ni=(if low=high then 0 else 1)} in
             add_test cij cinside ;
@@ -548,6 +626,49 @@ let rec pkey chan  = function
     end ;
     !r, !rc
 
+  (* Consider the following sequence of interval tests:
+
+       if a in [2; 10] then
+         if a in [2; 4] then act24
+         else if a in [5; 8] then act58
+         else act810
+       else act_default
+
+     Our interval check works by subtracting the interval lower
+     bound, then checking a range [0; n] using an unsigned
+     comparison. Naively we would generate code with one subtraction
+     to [a] before each comparison:
+
+       let tmp1 = a - 2 in
+       if tmp1 <=u 8 then
+         let tmp2 = a - 2 in
+         if tmp2 <=u 2 then act24
+         else
+           let tmp3 = a - 5 in
+           if tmp3 <=u 3 then act58
+           else act810
+       else act_default
+
+     but we can avoid some substractions by working with the result
+     of the first subtraction, instead of the original index [a],
+     inside the interval.
+
+       let a2 = a - 2 in
+       if a2 <=u 8 then
+         if a2 in <=u 2 then act24
+         else
+           let a5 = a2 - 3 in
+           if a5 <=u 3 then act58
+           else act810
+       else act_default
+
+     The type [t_ctx] represents an input argument "shifted" by a certain
+     (negative) offset by repeated substractions.
+
+     In the example above, [a5] would be represented with [off = -5].
+  *)
+  type 'a t_ctx =  {off : int ; arg : 'a}
+
   let make_if_test test arg i ifso ifnot =
     Arg.make_if
       (Arg.make_prim test [arg ; Arg.make_const i])
@@ -571,6 +692,12 @@ let rec pkey chan  = function
   and make_if_ne  arg i ifso ifnot =
     make_if_test Arg.neint arg i ifso ifnot
 
+  let make_if_nonzero arg ifso ifnot =
+    Arg.make_if (Arg.make_is_nonzero arg) ifso ifnot
+
+  let make_if_bool arg ifso ifnot =
+    Arg.make_if (Arg.arg_as_test arg) ifso ifnot
+
   let do_make_if_out h arg ifso ifno =
     Arg.make_if (Arg.make_isout h arg) ifso ifno
 
@@ -601,6 +728,7 @@ let rec pkey chan  = function
              do_make_if_in
                (Arg.make_const d) arg (mk_ifso ctx) (mk_ifno ctx))
 
+  (* Generate the code for a good test sequence. *)
   let rec c_test ctx ({cases=cases ; actions=actions} as s) =
     let lcases = Array.length cases in
     assert(lcases > 0) ;
@@ -609,7 +737,7 @@ let rec pkey chan  = function
 
     else begin
 
-      let w,_c = opt_count false cases in
+      let w,_c = opt_count cases in
 (*
   Printf.fprintf stderr
   "off=%d tactic=%a for %a\n"
@@ -619,8 +747,8 @@ let rec pkey chan  = function
       | No -> actions.(get_act cases 0) ctx
       | Inter (i,j) ->
           let low,high,inside, outside = coupe_inter i j cases in
-          let _,(cinside,_) = opt_count false inside
-          and _,(coutside,_) = opt_count false outside in
+          let _,(cinside,_) = opt_count inside
+          and _,(coutside,_) = opt_count outside in
           (* Costs are retrieved to put the code with more remaining tests
              in the privileged (positive) branch of ``if'' *)
           if low=high then begin
@@ -654,15 +782,20 @@ let rec pkey chan  = function
           end
       | Sep i ->
           let lim,left,right = coupe cases i in
-          let _,(cleft,_) = opt_count false left
-          and _,(cright,_) = opt_count false right in
+          let _,(cleft,_) = opt_count left
+          and _,(cright,_) = opt_count right in
           let left = {s with cases=left}
           and right = {s with cases=right} in
 
           if i=1 && (lim+ctx.off)=1 && get_low cases 0+ctx.off=0 then
-            Arg.make_if
-              ctx.arg
-              (c_test ctx right) (c_test ctx left)
+            if lcases = 2 && get_high cases 1+ctx.off = 1 then
+              make_if_bool
+                ctx.arg
+                (c_test ctx right) (c_test ctx left)
+            else
+              make_if_nonzero
+                ctx.arg
+                (c_test ctx right) (c_test ctx left)
           else if less_tests cright cleft then
             make_if_lt
               ctx.arg (lim+ctx.off)
@@ -675,13 +808,13 @@ let rec pkey chan  = function
     end
 
 
-  (* Minimal density of switches *)
-  let theta = ref 0.33333
+  (* Minimal density of dense switches. *)
+  let theta = 0.33333
 
-  (* Minimal number of tests to make a switch *)
-  let switch_min = ref 3
+  (* Minimal number of tests to make a dense switch. *)
+  let switch_min = 3
 
-  (* Particular case 0, 1, 2 *)
+  (* Particular case 0, 1, 2. *)
   let particular_case cases i j =
     j-i = 2 &&
     (let l1,_h1,act1 = cases.(i)
@@ -690,37 +823,37 @@ let rec pkey chan  = function
      l1+1=l2 && l2+1=l3 && l3=h3 &&
      act1 <> act3)
 
+  (* Approximation of the test sequence height,
+     used to determine cluster density. *)
   let approx_count cases i j =
     let l = j-i+1 in
-    if l < !cut then
-      let _,(_,{n=ntests}) = opt_count false (Array.sub cases i l) in
+    if l < small_size_limit then
+      (* on small input intervals, use test sequence height *)
+      let _,(_,{n=ntests}) = opt_count (Array.sub cases i l) in
       ntests
     else
+      (* otherwise use the standard notion of density
+         (number of non-default cases) *)
       l-1
 
-  (* Sends back a boolean that says whether is switch is worth or not *)
-
+  (* Sends back a boolean that says whether it is worth making a jump table. *)
   let dense {cases} i j =
     if i=j then true
     else
       let l,_,_ = cases.(i)
       and _,h,_ = cases.(j) in
-      let ntests =  approx_count cases i j in
+      let ntests = approx_count cases i j in
 (*
   (ntests+1) >= theta * (h-l+1)
 *)
       particular_case cases i j ||
-      (ntests >= !switch_min &&
+      ((* The switch_min test guarantees that we don't use jump tables
+          for very small switches. *)
+       ntests >= switch_min &&
        float_of_int ntests +. 1.0 >=
-       !theta *. (float_of_int h -. float_of_int l +. 1.0))
-
-  (* Compute clusters by dynamic programming
-     Adaptation of the correction to Bernstein
-     ``Correction to `Producing Good Code for the Case Statement' ''
-     S.K. Kannan and T.A. Proebsting
-     Software Practice and Experience Vol. 24(2) 233 (Feb 1994)
-  *)
+       theta *. (float_of_int h -. float_of_int l +. 1.0))
 
+  (* Compute an optimal clustering by dynamic programming. *)
   let comp_clusters s =
     let len = Array.length s.cases in
     let min_clusters = Array.make len max_int
@@ -740,8 +873,11 @@ let rec pkey chan  = function
     done ;
     min_clusters.(len-1),k
 
-  (* Assume j > i *)
+  (* The code to generate a dense switch is provided
+     by the functor parameter as Arg.make_switch
+     (which will typically use a jump table) *)
   let make_switch loc {cases=cases ; actions=actions} i j =
+    (* Assume j > i *)
     let ll,_,_ = cases.(i)
     and _,hh,_ = cases.(j) in
     let tbl = Array.make (hh-ll+1) 0
@@ -776,7 +912,7 @@ let rec pkey chan  = function
              (Arg.make_offset ctx.arg (-ll-ctx.off))
              (fun arg -> Arg.make_switch loc arg tbl acts))
 
-
+  (* Generate code from a clustering choice. *)
   let make_clusters loc ({cases=cases ; actions=actions} as s) n_clusters k =
     let len = Array.length cases in
     let r = Array.make n_clusters (0,0,0)
@@ -818,7 +954,6 @@ let rec pkey chan  = function
     let acts = Array.make !index (fun _ -> assert false) in
     Hashtbl.iter (fun _ (i,act) -> acts.(i) <- act) t ;
     {cases = r ; actions = acts}
-  ;;
 
 
   let do_zyva loc (low,high) arg cases actions =
@@ -851,12 +986,14 @@ let rec pkey chan  = function
         actions in
     !handlers,actions
 
+  (* Standard entry point. *)
   let zyva loc lh arg cases actions =
     assert (Array.length cases > 0) ;
     let actions = actions.act_get_shared () in
     let hs,actions = abstract_shared actions in
     hs (do_zyva loc lh arg cases actions)
 
+  (* Generate code using test sequences only, not Arg.make_switch *)
   and test_sequence arg cases actions =
     assert (Array.length cases > 0) ;
     let actions = actions.act_get_shared () in
@@ -873,6 +1010,5 @@ let rec pkey chan  = function
   prerr_endline "" ;
 *)
     hs (c_test {arg=arg ; off=0} s)
-  ;;
 
 end
