@@ -26,6 +26,8 @@ let convert (prim : Lambda.primitive) : Clambda_primitives.primitive =
   match prim with
   | Pmakeblock (tag, mutability, shape) ->
       Pmakeblock (tag, mutability, shape)
+  | Pmakelazyblock tag ->
+      Pmakelazyblock tag
   | Pfield (field, imm_or_pointer, mutability) ->
       Pfield (field, imm_or_pointer, mutability)
   | Pfield_computed -> Pfield_computed
@@ -144,11 +146,7 @@ let convert (prim : Lambda.primitive) : Clambda_primitives.primitive =
   | Pbigarraydim dim -> Pbigarraydim dim
   | Pbswap16 -> Pbswap16
   | Pint_as_pointer -> Pint_as_pointer
-  | Patomic_load { immediate_or_pointer } ->
-      Patomic_load { immediate_or_pointer }
-  | Patomic_exchange -> Patomic_exchange
-  | Patomic_cas -> Patomic_cas
-  | Patomic_fetch_add -> Patomic_fetch_add
+  | Patomic_load -> Patomic_load
   | Popaque -> Popaque
   | Pdls_get -> Pdls_get
   | Ppoll -> Ppoll
