@@ -67,7 +67,8 @@ type t =
   | Switch of Variable.t * switch
   | String_switch of Variable.t * (string * t) list * t option
   | Static_raise of Static_exception.t * Variable.t list
-  | Static_catch of Static_exception.t * Variable.t list * t * t
+  | Static_catch of
+      Static_exception.t * (Variable.t * Lambda.value_kind) list * t * t
   | Try_with of t * Variable.t * t
   | While of t * t
   | For of for_loop
@@ -188,6 +189,11 @@ let print_project_closure = Projection.print_project_closure
 
 (** CR-someday lwhite: use better name than this *)
 let rec lam ppf (flam : t) =
+  let print_kind ppf (kind : Lambda.value_kind) =
+    match kind with
+    | Pgenval -> ()
+    | _ -> Format.fprintf ppf " %a" Printlambda.value_kind kind
+  in
   match flam with
   | Var (id) ->
       Variable.print ppf id
@@ -240,11 +246,6 @@ let rec lam ppf (flam : t) =
       let expr = letbody body in
       fprintf ppf ")@]@ %a)@]" lam expr
   | Let_mutable { var = mut_var; initial_value = var; body; contents_kind } ->
-    let print_kind ppf (kind : Lambda.value_kind) =
-      match kind with
-      | Pgenval -> ()
-      | _ -> Format.fprintf ppf " %a" Printlambda.value_kind kind
-    in
     fprintf ppf "@[<2>(let_mutable%a@ @[<2>%a@ %a@]@ %a)@]"
       print_kind contents_kind
       Mutable_variable.print mut_var
@@ -302,7 +303,8 @@ let rec lam ppf (flam : t) =
            | [] -> ()
            | _ ->
                List.iter
-                 (fun x -> fprintf ppf " %a" Variable.print x)
+                 (fun (x, kind) ->
+                    fprintf ppf " %a%a" Variable.print x print_kind kind)
                  vars)
         vars
         lam lhandler
@@ -554,7 +556,7 @@ let rec variables_usage ?ignore_uses_as_callee ?ignore_uses_as_argument
       | Static_raise (_, es) ->
         List.iter free_variable es
       | Static_catch (_, vars, e1, e2) ->
-        List.iter bound_variable vars;
+        List.iter (fun (v, _) ->  bound_variable v) vars;
         aux e1;
         aux e2
       | Try_with (e1, var, e2) ->
