@@ -572,6 +572,7 @@ class man =
              bs b (if r.rf_mutable then "\n\n.B mutable \n" else "\n ");
              bs b (r.rf_name^" : ");
              self#man_of_type_expr b father r.rf_type;
+             bs b (if r.rf_atomic then "\n.B [@atomic] " else "");
              bs b ";";
              self#field_comment b r.rf_text ;
            ) l;
@@ -846,6 +847,7 @@ class man =
       if f.rf_mutable then bs b (Odoc_messages.mutab^" ");
       bs b (f.rf_name^" : ");
       self#man_of_type_expr b modname f.rf_type;
+      if f.rf_atomic then bs b "\n.B [@atomic] ";
       bs b "\n.sp\n";
       self#man_of_info b f.rf_text;
       bs b "\n.sp\n"
