@@ -13,15 +13,12 @@
 (*                                                                        *)
 (**************************************************************************)
 
-module String = Misc.Stdlib.String
-
 (** Representation and manipulation of modules and module types. *)
 
-let print_DEBUG s = print_string s ; print_newline ()
+module String = Misc.Stdlib.String
 
 module Name = Odoc_name
 
-(** To keep the order of elements in a module. *)
 type module_element =
     Element_module of t_module
   | Element_module_type of t_module_type
@@ -33,8 +30,6 @@ type module_element =
   | Element_exception of Odoc_exception.t_exception
   | Element_type of Odoc_type.t_type
   | Element_module_comment of Odoc_types.text
-
-(** Used where we can reference t_module or t_module_type *)
 and mmt =
   | Mod of t_module
   | Modtype of t_module_type
@@ -57,18 +52,17 @@ and module_parameter = {
     mp_kind : module_type_kind ; (** the way the parameter was built *)
   }
 
-(** Different kinds of module. *)
 and module_kind =
   | Module_struct of module_element list
   | Module_alias of module_alias (** complete name and corresponding module if we found it *)
   | Module_functor of module_parameter * module_kind
   | Module_apply of module_kind * module_kind
+  | Module_apply_unit of module_kind
   | Module_with of module_type_kind * string
   | Module_constraint of module_kind * module_type_kind
   | Module_typeof of string (** by now only the code of the module expression *)
   | Module_unpack of string * module_type_alias (** code of the expression and module type alias *)
 
-(** Representation of a module. *)
 and t_module = {
     m_name : Name.t ;
     mutable m_type : Types.module_type ;
@@ -88,7 +82,6 @@ and module_type_alias = {
     mutable mta_module : t_module_type option ; (** the real module type if we could associate it *)
   }
 
-(** Different kinds of module type. *)
 and module_type_kind =
   | Module_type_struct of module_element list
   | Module_type_functor of module_parameter * module_type_kind
@@ -96,7 +89,6 @@ and module_type_kind =
   | Module_type_with of module_type_kind * string (** the module type kind and the code of the with constraint *)
   | Module_type_typeof of string (** by now only the code of the module expression *)
 
-(** Representation of a module type. *)
 and t_module_type = {
     mt_name : Name.t ;
     mutable mt_info : Odoc_types.info option ;
@@ -108,10 +100,6 @@ and t_module_type = {
     mutable mt_loc : Odoc_types.location ;
   }
 
-
-(** {1 Functions} *)
-
-(** Returns the list of values from a list of module_element. *)
 let values l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -122,7 +110,6 @@ let values l =
     []
     l
 
-(** Returns the list of types from a list of module_element. *)
 let types l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -133,7 +120,6 @@ let types l =
     []
     l
 
-(** Returns the list of type extensions from a list of module_element. *)
 let type_extensions l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -144,7 +130,6 @@ let type_extensions l =
     []
     l
 
-(** Returns the list of exceptions from a list of module_element. *)
 let exceptions l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -155,7 +140,6 @@ let exceptions l =
     []
     l
 
-(** Returns the list of classes from a list of module_element. *)
 let classes l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -166,7 +150,6 @@ let classes l =
     []
     l
 
-(** Returns the list of class types from a list of module_element. *)
 let class_types l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -177,7 +160,6 @@ let class_types l =
     []
     l
 
-(** Returns the list of modules from a list of module_element. *)
 let modules l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -188,7 +170,6 @@ let modules l =
     []
     l
 
-(** Returns the list of module types from a list of module_element. *)
 let mod_types l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -199,7 +180,6 @@ let mod_types l =
     []
     l
 
-(** Returns the list of module comment from a list of module_element. *)
 let comments l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -210,7 +190,6 @@ let comments l =
     []
     l
 
-(** Returns the list of included modules from a list of module_element. *)
 let included_modules l =
   List.fold_left
     (fun acc -> fun ele ->
@@ -221,8 +200,6 @@ let included_modules l =
     []
     l
 
-(** Returns the list of elements of a module type.
-   @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let rec module_type_elements ?(trans=true) mt =
   let rec iter_kind = function
     | None -> []
@@ -244,20 +221,14 @@ let rec module_type_elements ?(trans=true) mt =
   in
   iter_kind mt.mt_kind
 
-(** Returns the list of elements of a module.
-   @param trans indicates if, for aliased modules, we must perform a transitive search.
-*)
 let module_elements ?(trans=true) m =
 (* visited is used to guard against aliases loop
      (e.g [module rec M:sig end=M] induced loop.
 *)
   let rec module_elements visited ?(trans=true) m =
     let rec iter_kind = function
-        Module_struct l ->
-          print_DEBUG "Odoc_module.module_elements: Module_struct";
-          l
+        Module_struct l -> l
       | Module_alias ma ->
-          print_DEBUG "Odoc_module.module_elements: Module_alias";
           if trans then
             match ma.ma_module with
               None -> []
@@ -270,18 +241,15 @@ let module_elements ?(trans=true) m =
           else
             []
       | Module_functor (_, k)
-      | Module_apply (k, _) ->
-          print_DEBUG "Odoc_module.module_elements: Module_functor ou Module_apply";
-          iter_kind k
+      | Module_apply (k, _) -> iter_kind k
+      | Module_apply_unit k -> iter_kind k
       | Module_with (tk,_) ->
-          print_DEBUG "Odoc_module.module_elements: Module_with";
           module_type_elements ~trans: trans
             { mt_name = "" ; mt_info = None ; mt_type = None ;
               mt_is_interface = false ; mt_file = "" ; mt_kind = Some tk ;
               mt_loc = Odoc_types.dummy_loc ;
             }
       | Module_constraint (k, _tk) ->
-          print_DEBUG "Odoc_module.module_elements: Module_constraint";
           (* FIXME : use k or tk ? *)
           module_elements visited ~trans: trans
             { m_name = "" ;
@@ -306,8 +274,6 @@ let module_elements ?(trans=true) m =
     iter_kind m.m_kind in
   module_elements String.Set.empty ~trans m
 
-(** Returns the list of values of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_values ?(trans=true) m = values (module_elements ~trans m)
 
 (** Returns the list of functional values of a module.
@@ -317,51 +283,29 @@ let module_functions ?(trans=true) m =
     (fun v -> Odoc_value.is_function v)
     (values (module_elements ~trans m))
 
-(** Returns the list of non-functional values of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_simple_values ?(trans=true) m =
     List.filter
     (fun v -> not (Odoc_value.is_function v))
     (values (module_elements ~trans m))
 
-(** Returns the list of types of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_types ?(trans=true) m = types (module_elements ~trans m)
 
-(** Returns the list of type extensions of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_extensions ?(trans=true) m = type_extensions (module_elements ~trans m)
 
-(** Returns the list of exceptions of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_exceptions ?(trans=true) m = exceptions (module_elements ~trans m)
 
-(** Returns the list of classes of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_classes ?(trans=true) m = classes (module_elements ~trans m)
 
-(** Returns the list of class types of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_class_types ?(trans=true) m = class_types (module_elements ~trans m)
 
-(** Returns the list of modules of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_modules ?(trans=true) m = modules (module_elements ~trans m)
 
-(** Returns the list of module types of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_module_types ?(trans=true) m = mod_types (module_elements ~trans m)
 
-(** Returns the list of included module of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_included_modules ?(trans=true) m = included_modules (module_elements ~trans m)
 
-(** Returns the list of comments of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_comments ?(trans=true) m = comments (module_elements ~trans m)
 
-(** Access to the parameters, for a functor type.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let rec module_type_parameters ?(trans=true) mt =
   let rec iter k =
     match k with
@@ -400,8 +344,6 @@ let rec module_type_parameters ?(trans=true) mt =
   in
   iter mt.mt_kind
 
-(** Access to the parameters, for a functor.
-   @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 and module_parameters ?(trans=true) m =
   let rec iter = function
       Module_functor (p, k) ->
@@ -435,14 +377,13 @@ and module_parameters ?(trans=true) m =
             mt_loc = Odoc_types.dummy_loc }
     | Module_struct _
     | Module_apply _
+    | Module_apply_unit _
     | Module_with _
     | Module_typeof _
     | Module_unpack _ -> []
   in
   iter m.m_kind
 
-(** access to all submodules and submodules of submodules ... of the given module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let rec module_all_submodules ?(trans=true) m =
   let l = module_modules ~trans m in
   List.fold_left
@@ -450,7 +391,6 @@ let rec module_all_submodules ?(trans=true) m =
     l
     l
 
-(** The module type is a functor if it is defined as a functor or if it is an alias for a functor. *)
 let rec module_type_is_functor mt =
   let rec iter k =
     match k with
@@ -469,7 +409,6 @@ let rec module_type_is_functor mt =
   in
   iter mt.mt_kind
 
-(** The module is a functor if it is defined as a functor or if it is an alias for a functor. *)
 let module_is_functor m =
   let rec iter visited = function
       Module_functor _ -> true
@@ -488,64 +427,36 @@ let module_is_functor m =
   in
   iter String.Set.empty m.m_kind
 
-(** Returns the list of values of a module type.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_values ?(trans=true) m = values (module_type_elements ~trans m)
 
-(** Returns the list of types of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_types ?(trans=true) m = types (module_type_elements ~trans m)
 
-(** Returns the list of type extensions of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_type_extensions ?(trans=true) m = type_extensions (module_type_elements ~trans m)
 
-(** Returns the list of exceptions of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_exceptions ?(trans=true) m = exceptions (module_type_elements ~trans m)
 
-(** Returns the list of classes of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_classes ?(trans=true) m = classes (module_type_elements ~trans m)
 
-(** Returns the list of class types of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_class_types ?(trans=true) m = class_types (module_type_elements ~trans m)
 
-(** Returns the list of modules of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_modules ?(trans=true)  m = modules (module_type_elements ~trans m)
 
-(** Returns the list of module types of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_module_types ?(trans=true) m = mod_types (module_type_elements ~trans m)
 
-(** Returns the list of included module of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_included_modules ?(trans=true) m = included_modules (module_type_elements ~trans m)
 
-(** Returns the list of comments of a module.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_comments ?(trans=true) m = comments (module_type_elements ~trans m)
 
-(** Returns the list of functional values of a module type.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_functions ?(trans=true) mt =
   List.filter
     (fun v -> Odoc_value.is_function v)
     (values (module_type_elements ~trans mt))
 
-(** Returns the list of non-functional values of a module type.
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_type_simple_values ?(trans=true) mt =
     List.filter
     (fun v -> not (Odoc_value.is_function v))
     (values (module_type_elements ~trans mt))
 
-(** {1 Functions for modules and module types} *)
-
-(** The list of classes defined in this module and all its modules, functors, ....
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let rec module_all_classes ?(trans=true) m =
   List.fold_left
     (fun acc -> fun m -> acc @ (module_all_classes ~trans m))
@@ -557,8 +468,6 @@ let rec module_all_classes ?(trans=true) m =
     )
     (module_modules ~trans m)
 
-(** The list of classes defined in this module type and all its modules, functors, ....
-  @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 and module_type_all_classes ?(trans=true) mt =
   List.fold_left
     (fun acc -> fun m -> acc @ (module_all_classes ~trans m))
