@@ -15,8 +15,9 @@
 
 (** The functions to get a string from different kinds of elements (types, modules, ...). *)
 
-(** @return the variance string for the given type and (covariant, contravariant) information. *)
-val string_of_variance : Odoc_type.t_type -> (bool * bool) -> string
+(** @return the variance and injectivity annotation for the given type and
+    variance and injectivity information. *)
+val string_of_variance : Odoc_type.t_type -> Types.Variance.t -> string
 
 (** This function returns a string to represent the given list of types,
    with a given separator.
