@@ -1,19 +1,21 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                             OCamldoc                                *)
-(*                                                                     *)
-(*      Olivier Andrieu, base sur du code de Maxence Guesdon           *)
-(*                                                                     *)
-(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the Q Public License version 1.0.               *)
-(*                                                                     *)
-(***********************************************************************)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*       Olivier Andrieu, base sur du code de Maxence Guesdon             *)
+(*                                                                        *)
+(*   Copyright 2001 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (** Generation of Texinfo documentation. *)
 
 open Odoc_info
-open Parameter
 open Value
 open Type
 open Extension
@@ -27,7 +29,7 @@ let info_section = ref "OCaml"
 
 let info_entry = ref []
 
-(** {2 Some small helper functions} *)
+(** {1 Some small helper functions} *)
 
 let puts_nl chan s =
   output_string chan s ;
@@ -139,7 +141,7 @@ let indices_names = [
 (** Module for generating various Texinfo things (menus, xrefs, ...) *)
 module Texi =
 struct
-  (** Associations of strings to subsitute in Texinfo code. *)
+  (** Associations of strings to substitute in Texinfo code. *)
   let subst_strings = [
     (Str.regexp "@", "@@") ;
     (Str.regexp "{", "@{") ;
@@ -238,39 +240,36 @@ end
 
 
 
-(** {2 Generation of Texinfo code} *)
+(** {1 Generation of Texinfo code} *)
+
+(** {2 Associations between a title number and texinfo code.} *)
+let titles_and_headings = ref [
+    0, ("@chapter ", "@majorheading ")  ;
+    1, ("@chapter ", "@majorheading ")  ;
+    2, ("@section ", "@heading ")  ;
+    3, ("@subsection ", "@subheading ") ;
+    4, ("@subsubsection ", "@subsubheading ")  ;
+  ]
+
+let title = fst
+let heading = snd
+
+let fallback_title =
+  "@unnumberedsubsubsec "
+
+let fallback_heading =
+  "@subsubheading "
 
 (** This class generates Texinfo code from text structures *)
 class text =
   object(self)
 
-  (** Associations between a title number and texinfo code. *)
-    val titles = [
-      1, "@chapter " ;
-      2, "@section " ;
-      3, "@subsection " ;
-      4, "@subsubsection " ;
-    ]
-
-    val fallback_title =
-      "@unnumberedsubsubsec "
-
-    val headings = [
-      1, "@majorheading " ;
-      2, "@heading " ;
-      3, "@subheading " ;
-      4, "@subsubheading " ;
-    ]
-
-    val fallback_heading =
-      "@subsubheading "
-
     method escape =
       Texi.escape
 
     (** this method is not used here but is virtual
         in a class we will inherit later *)
-    method label ?(no_ : bool option) (_ : string) : string =
+    method label ?no_:(_ : bool option) (_ : string) : string =
       failwith "gni"
 
     (** Return the Texinfo code corresponding to the [text] parameter.*)
@@ -279,7 +278,7 @@ class text =
         (List.map self#texi_of_text_element t)
 
 
-    (** {3 Conversion methods}
+    (** {2 Conversion methods}
        [texi_of_????] converts a [text_element] to a Texinfo string. *)
 
     (** Return the Texinfo code for the [text_element] in parameter. *)
@@ -308,10 +307,10 @@ class text =
       | Odoc_info.Custom (s,t) -> self#texi_of_custom_text s t
       | Odoc_info.Target (target, code) -> self#texi_of_Target ~target ~code
 
-    method texi_of_custom_text s t = ""
+    method texi_of_custom_text _ _ = ""
 
     method texi_of_Target ~target ~code =
-      if String.lowercase target = "texi" then code else ""
+      if String.lowercase_ascii target = "texi" then code else ""
 
     method texi_of_Verbatim s = s
     method texi_of_Raw s = self#escape s
@@ -348,7 +347,7 @@ class text =
         [ "@format" ; self#texi_of_text t ; "@end format" ; "" ]
     method texi_of_Title n t =
       let t_begin =
-        try List.assoc n titles
+        try title @@ List.assoc n !titles_and_headings
         with Not_found -> fallback_title in
       t_begin ^ (self#texi_of_text t) ^ "\n"
     method texi_of_Link s t =
@@ -375,7 +374,7 @@ class text =
 
     method heading n t =
       let f =
-        try List.assoc n headings
+        try heading @@ List.assoc n !titles_and_headings
         with Not_found -> fallback_heading
       in
       f ^ (self#texi_of_text t) ^ "\n"
@@ -394,10 +393,10 @@ struct
     Texinfo documentation. *)
 class texi =
   object (self)
-    inherit text as to_texi
+    inherit text
     inherit Odoc_to_text.to_text as to_text
 
-    (** {3 Small helper stuff.} *)
+    (** {2 Small helper stuff.} *)
 
     val maxdepth = 4
 
@@ -451,7 +450,7 @@ class texi =
             | Raw s -> Raw (Str.global_replace re rep s)
             | txt -> txt) t
 
-    (** {3 [text] values generation}
+    (** {2 [text] values generation}
        Generates [text] values out of description parts.
        Redefines some of methods of {! Odoc_to_text.to_text}. *)
 
@@ -473,7 +472,7 @@ class texi =
                    Raw " " ; Raw s ] @ t @ [ Newline ])
            see_l)
 
-    method text_of_before l =
+    method! text_of_before l =
       List.flatten
       (List.map
         (fun x -> linebreak :: (to_text#text_of_before [x])) l)
@@ -530,7 +529,7 @@ class texi =
                  [ ( match info.i_deprecated with
                  | None -> []
                  | Some t ->
-                     (Raw (Odoc_messages.deprecated ^ " ")) ::
+                     (Raw (Odoc_messages.deprecated ^ ". ")) ::
                      (self#fix_linebreaks t)
                      @ [ Newline ; Newline ] ) ;
                    self#text_of_desc info.i_desc ;
@@ -554,6 +553,7 @@ class texi =
                    then ( linebreak ::
                           self#text_of_return_opt info.i_return_value )
                    else [] ;
+                   self#text_of_alerts info.i_alerts ;
                    self#text_of_custom info.i_custom ;
                  ] in
           if block
@@ -563,7 +563,7 @@ class texi =
     method texi_of_info i =
       self#texi_of_text (self#text_of_info i)
 
-    (** {3 Conversion of [module_elements] into Texinfo strings}
+    (** {2 Conversion of [module_elements] into Texinfo strings}
        The following functions convert [module_elements] and their
        description to [text] values then to Texinfo strings using the
        functions above. *)
@@ -625,25 +625,28 @@ class texi =
 
 
     method string_of_type_parameters t =
-      let f (tp, co, cn) =
+      let f (tp, v) =
         Printf.sprintf "%s%s"
-          (Odoc_info.string_of_variance t (co, cn))
+          (Odoc_info.string_of_variance t v)
           (Odoc_info.string_of_type_expr tp)
       in
       match t.ty_parameters with
       | [] -> ""
-      | [ (tp, co, cn) ] ->
-          (f (tp, co, cn))^" "
+      | [ tv ] -> (f tv)^" "
       | l ->
           Printf.sprintf "(%s) "
             (String.concat ", " (List.map f l))
 
-    method string_of_type_args (args:Types.type_expr list) (ret:Types.type_expr option) =
+    method string_of_type_args (args:constructor_args) (ret:Types.type_expr option) =
+      let f = function
+        | Cstr_tuple l -> Odoc_info.string_of_type_list " * " l
+        | Cstr_record l -> Odoc_info.string_of_record l
+      in
       match args, ret with
-      | [], None -> ""
-      | args, None -> " of " ^ (Odoc_info.string_of_type_list " * " args)
-      | [], Some r -> " : " ^ (Odoc_info.string_of_type_expr r)
-      | args, Some r -> " : " ^ (Odoc_info.string_of_type_list " * " args) ^
+      | Cstr_tuple [], None -> ""
+      | args, None -> " of " ^ (f args)
+      | Cstr_tuple [], Some r -> " : " ^ (Odoc_info.string_of_type_expr r)
+      | args, Some r -> " : " ^ (f args) ^
                                 " -> " ^ (Odoc_info.string_of_type_expr r)
 
     (** Return Texinfo code for a type. *)
@@ -879,7 +882,7 @@ class texi =
       self#texi_of_text t
 
     (** Return the Texinfo code for the given class element. *)
-    method texi_of_class_element class_name class_ele =
+    method texi_of_class_element _class_name class_ele =
       match class_ele with
       | Class_attribute att -> self#texi_of_attribute att
       | Class_method met -> self#texi_of_method met
@@ -901,7 +904,7 @@ class texi =
           self#texi_of_text (Newline :: t @ [Newline])
       )
 
-    (** {3 Generating methods }
+    (** {2 Generating methods }
        These methods write Texinfo code to an [out_channel] *)
 
     (** Generate the Texinfo code for the given list of inherited classes.*)
