@@ -21,6 +21,8 @@ val files_to_test : string list
 
 val promote : bool
 
+val default_timeout : int
+
 val usage : string
 
 val find_test_dirs : string list
@@ -28,3 +30,9 @@ val find_test_dirs : string list
 val list_tests : string list
 
 val keep_test_dir_on_success : bool
+
+val show_timings : bool
+
+val translate : bool
+val style : Translate.style
+val compact : bool
