@@ -18,9 +18,12 @@
 type t = {
   test_name : string;
   test_run_by_default : bool;
-  test_actions : Actions.t list
+  test_actions : Actions.t list;
+  test_description : string
 }
 
+val null : t
+
 val compare : t -> t -> int
 
 val register : t -> unit
