@@ -35,6 +35,14 @@ type tsl_item =
 
 type tsl_block = tsl_item list
 
+type t = Ast of tsl_item list * t list
+
+let rec split_env l =
+  match l with
+  | Environment_statement env :: tl ->
+    let (env2, rest) = split_env tl in (env :: env2, rest)
+  | _ -> ([], l)
+
 let make ?(loc = Location.none) foo = { node = foo; loc = loc }
 
 let make_identifier = make
