@@ -41,10 +41,6 @@
 #include <wtypes.h>
 #else
 #include <sys/mman.h>
-#endif
-
-
-#if defined(HAS_UNISTD)
 #include <unistd.h>
 #endif
 
@@ -465,7 +461,7 @@ caml_runtime_events_read_poll(struct caml_runtime_events_cursor *cursor,
         return E_CORRUPT_STREAM;
     }
 
-    struct runtime_events_buffer_header *runtime_events_buffer_header =
+    const struct runtime_events_buffer_header *runtime_events_buffer_header =
         (struct runtime_events_buffer_header *)(
           get_map_offset(cursor, cursor->metadata.headers_offset,
                          domain_num,
@@ -1338,3 +1334,11 @@ CAMLprim value caml_ml_runtime_events_read_poll(value wrapper,
 
   CAMLreturn(Val_int(events_consumed));
 }
+
+CAMLprim uint64_t caml_ml_runtime_current_timestamp_unboxed(value unit) {
+  return caml_time_counter();
+}
+
+CAMLprim value caml_ml_runtime_current_timestamp(value unit) {
+  return caml_copy_int64(caml_time_counter());
+}
