@@ -22,17 +22,10 @@
 #include <pthread.h>
 #include <signal.h>
 #include <time.h>
-#ifdef HAS_UNISTD
+#ifndef _WIN32
 #include <unistd.h>
 #endif
 
-typedef int st_retcode;
-
-/* OS-specific initialization */
-static int st_initialize(void)
-{
-  return 0;
-}
 
 typedef pthread_t st_thread_id;
 
@@ -87,10 +80,10 @@ Caml_inline void st_tls_set(st_tlskey k, void * v)
    threads. */
 
 typedef struct {
-  int init;                       /* have the mutex and the cond been
+  bool init;                      /* have the mutex and the cond been
                                      initialized already? */
   pthread_mutex_t lock;           /* to protect contents */
-  uintnat busy;                   /* 0 = free, 1 = taken */
+  bool busy;                      /* false = free, true = taken */
   atomic_uintnat waiters;         /* number of threads waiting on master lock */
   pthread_cond_t is_free;         /* signaled when free */
 } st_masterlock;
@@ -104,9 +97,9 @@ static int st_masterlock_init(st_masterlock * m)
     if (rc != 0) goto out_err;
     rc = pthread_cond_init(&m->is_free, NULL);
     if (rc != 0) goto out_err2;
-    m->init = 1;
+    m->init = true;
   }
-  m->busy = 1;
+  m->busy = true;
   atomic_store_release(&m->waiters, 0);
   return 0;
 
@@ -121,35 +114,6 @@ static uintnat st_masterlock_waiters(st_masterlock * m)
   return atomic_load_acquire(&m->waiters);
 }
 
-static void st_bt_lock_acquire(st_masterlock *m) {
-
-  /* We do not want to signal the backup thread if it is not "working"
-     as it may very well not be, because we could have just resumed
-     execution from another thread right away. */
-  if (caml_bt_is_in_blocking_section()) {
-    caml_bt_enter_ocaml();
-  }
-
-  caml_acquire_domain_lock();
-
-  return;
-}
-
-static void st_bt_lock_release(st_masterlock *m) {
-
-  /* Here we do want to signal the backup thread iff there's
-     no thread waiting to be scheduled, and the backup thread is currently
-     idle. */
-  if (st_masterlock_waiters(m) == 0 &&
-      caml_bt_is_in_blocking_section() == 0) {
-    caml_bt_exit_ocaml();
-  }
-
-  caml_release_domain_lock();
-
-  return;
-}
-
 static void st_masterlock_acquire(st_masterlock *m)
 {
   pthread_mutex_lock(&m->lock);
@@ -158,8 +122,8 @@ static void st_masterlock_acquire(st_masterlock *m)
     pthread_cond_wait(&m->is_free, &m->lock);
     atomic_fetch_add(&m->waiters, -1);
   }
-  m->busy = 1;
-  st_bt_lock_acquire(m);
+  m->busy = true;
+  st_bt_lock_acquire();
   pthread_mutex_unlock(&m->lock);
 
   return;
@@ -168,8 +132,8 @@ static void st_masterlock_acquire(st_masterlock *m)
 static void st_masterlock_release(st_masterlock * m)
 {
   pthread_mutex_lock(&m->lock);
-  m->busy = 0;
-  st_bt_lock_release(m);
+  m->busy = false;
+  st_bt_lock_release(st_masterlock_waiters(m) == 0);
   pthread_cond_signal(&m->is_free);
   pthread_mutex_unlock(&m->lock);
 
@@ -200,7 +164,7 @@ Caml_inline void st_thread_yield(st_masterlock * m)
     return;
   }
 
-  m->busy = 0;
+  m->busy = false;
   atomic_fetch_add(&m->waiters, +1);
   pthread_cond_signal(&m->is_free);
   /* releasing the domain lock but not triggering bt messaging
@@ -217,7 +181,7 @@ Caml_inline void st_thread_yield(st_masterlock * m)
        pthread_cond_wait(&m->is_free, &m->lock);
   } while (m->busy);
 
-  m->busy = 1;
+  m->busy = true;
   atomic_fetch_add(&m->waiters, -1);
 
   caml_acquire_domain_lock();
@@ -231,7 +195,7 @@ Caml_inline void st_thread_yield(st_masterlock * m)
 
 typedef struct st_event_struct {
   pthread_mutex_t lock;         /* to protect contents */
-  int status;                   /* 0 = not triggered, 1 = triggered */
+  bool status;                  /* false = not triggered, true = triggered */
   pthread_cond_t triggered;     /* signaled when triggered */
 } * st_event;
 
@@ -246,7 +210,7 @@ static int st_event_create(st_event * res)
   rc = pthread_cond_init(&e->triggered, NULL);
   if (rc != 0)
   { pthread_mutex_destroy(&e->lock); caml_stat_free(e); return rc; }
-  e->status = 0;
+  e->status = false;
   *res = e;
   return 0;
 }
@@ -265,7 +229,7 @@ static int st_event_trigger(st_event e)
   int rc;
   rc = pthread_mutex_lock(&e->lock);
   if (rc != 0) return rc;
-  e->status = 1;
+  e->status = true;
   rc = pthread_mutex_unlock(&e->lock);
   if (rc != 0) return rc;
   rc = pthread_cond_broadcast(&e->triggered);
@@ -277,36 +241,10 @@ static int st_event_wait(st_event e)
   int rc;
   rc = pthread_mutex_lock(&e->lock);
   if (rc != 0) return rc;
-  while(e->status == 0) {
+  while(!e->status) {
     rc = pthread_cond_wait(&e->triggered, &e->lock);
     if (rc != 0) return rc;
   }
   rc = pthread_mutex_unlock(&e->lock);
   return rc;
 }
-
-struct caml_thread_tick_args {
-  int domain_id;
-  atomic_uintnat* stop;
-};
-
-/* The tick thread: interrupt the domain periodically to force preemption  */
-static void * caml_thread_tick(void * arg)
-{
-  struct caml_thread_tick_args* tick_thread_args =
-    (struct caml_thread_tick_args*) arg;
-  int domain_id = tick_thread_args->domain_id;
-  atomic_uintnat* stop = tick_thread_args->stop;
-  caml_stat_free(tick_thread_args);
-
-  caml_init_domain_self(domain_id);
-  caml_domain_state *domain = Caml_state;
-
-  while(! atomic_load_acquire(stop)) {
-    st_msleep(Thread_timeout);
-
-    atomic_store_release(&domain->requested_external_interrupt, 1);
-    caml_interrupt_self();
-  }
-  return NULL;
-}
