@@ -21,7 +21,7 @@
 #include <caml/io.h>
 #include <caml/signals.h>
 #include "caml/unixsupport.h"
-#ifdef HAS_UNISTD
+#ifndef _WIN32
 #include <unistd.h>
 #endif
 
