@@ -19,9 +19,9 @@
 #include <caml/alloc.h>
 #include <caml/fail.h>
 #include <caml/osdeps.h>
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 
-#if !defined (_WIN32) && !macintosh
+#if !defined(_WIN32)
 #include <sys/param.h>
 #endif
 
@@ -35,18 +35,18 @@
 
 #ifdef HAS_GETCWD
 
-CAMLprim value unix_getcwd(value unit)
+CAMLprim value caml_unix_getcwd(value unit)
 {
   char_os buff[PATH_MAX];
   char_os * ret;
   ret = getcwd_os(buff, sizeof(buff)/sizeof(*buff));
-  if (ret == 0) uerror("getcwd", Nothing);
+  if (ret == 0) caml_uerror("getcwd", Nothing);
   return caml_copy_string_of_os(buff);
 }
 
 #else
 
-CAMLprim value unix_getcwd(value unit)
+CAMLprim value caml_unix_getcwd(value unit)
 { caml_invalid_argument("getcwd not implemented"); }
 
 #endif
