@@ -14,9 +14,9 @@
 /**************************************************************************/
 
 #include <caml/mlvalues.h>
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 
-CAMLprim value unix_getgid(value unit)
+CAMLprim value caml_unix_getgid(value unit)
 {
   return Val_int(getgid());
 }
