@@ -19,7 +19,7 @@
 #ifndef _WIN32
 #include <sys/param.h>
 #endif
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 
 #ifdef HAS_GETHOSTNAME
 
@@ -27,12 +27,12 @@
 #define MAXHOSTNAMELEN 256
 #endif
 
-CAMLprim value unix_gethostname(value unit)
+CAMLprim value caml_unix_gethostname(value unit)
 {
   char name[MAXHOSTNAMELEN];
   gethostname(name, MAXHOSTNAMELEN);
   name[MAXHOSTNAMELEN-1] = 0;
-  return copy_string(name);
+  return caml_copy_string(name);
 }
 
 #else
@@ -40,7 +40,7 @@ CAMLprim value unix_gethostname(value unit)
 
 #include <sys/utsname.h>
 
-CAMLprim value unix_gethostname(value unit)
+CAMLprim value caml_unix_gethostname(value unit)
 {
   struct utsname un;
   uname(&un);
@@ -49,8 +49,8 @@ CAMLprim value unix_gethostname(value unit)
 
 #else
 
-CAMLprim value unix_gethostname(value unit)
-{ invalid_argument("gethostname not implemented"); }
+CAMLprim value caml_unix_gethostname(value unit)
+{ caml_invalid_argument("gethostname not implemented"); }
 
 #endif
 #endif
