@@ -13,24 +13,31 @@
 /*                                                                        */
 /**************************************************************************/
 
+#ifndef _WIN32
 #include <sys/types.h>
 #include <sys/stat.h>
+#endif
+
+#define CAML_INTERNALS
 #include <caml/mlvalues.h>
+#include <caml/osdeps.h>
+#include <caml/misc.h>
 #include <caml/memory.h>
 #include <caml/signals.h>
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 
-CAMLprim value unix_mkdir(value path, value perm)
+CAMLprim value caml_unix_mkdir(value path, value vperm)
 {
-  CAMLparam2(path, perm);
-  char * p;
+  CAMLparam2(path, vperm);
+  char_os * p;
   int ret;
+  int perm = Int_val(vperm);
   caml_unix_check_path(path, "mkdir");
-  p = caml_stat_strdup(String_val(path));
+  p = caml_stat_strdup_to_os(String_val(path));
   caml_enter_blocking_section();
-  ret = mkdir(p, Int_val(perm));
+  ret = mkdir_os(p, perm);
   caml_leave_blocking_section();
   caml_stat_free(p);
-  if (ret == -1) uerror("mkdir", path);
+  if (ret == -1) caml_uerror("mkdir", path);
   CAMLreturn(Val_unit);
 }
