@@ -29,7 +29,7 @@
 #include "caml/unixsupport.h"
 
 #include <errno.h>
-#ifdef HAS_UNISTD
+#ifndef _WIN32
 #include <unistd.h>
 #endif
 #ifdef HAS_MMAP
