@@ -20,7 +20,7 @@
 #include <caml/signals.h>
 #include "caml/unixsupport.h"
 #include <string.h>
-#ifdef HAS_UNISTD
+#ifndef _WIN32
 #include <unistd.h>
 #endif
 #include <fcntl.h>
