@@ -19,9 +19,9 @@
 #include <caml/memory.h>
 #include <caml/signals.h>
 #include <caml/osdeps.h>
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 
-CAMLprim value unix_rmdir(value path)
+CAMLprim value caml_unix_rmdir(value path)
 {
   CAMLparam1(path);
   char_os * p;
@@ -32,6 +32,6 @@ CAMLprim value unix_rmdir(value path)
   ret = rmdir_os(p);
   caml_leave_blocking_section();
   caml_stat_free(p);
-  if (ret == -1) uerror("rmdir", path);
+  if (ret == -1) caml_uerror("rmdir", path);
   CAMLreturn(Val_unit);
 }
