@@ -15,15 +15,17 @@
 
 #include <caml/fail.h>
 #include <caml/mlvalues.h>
-#include "unixsupport.h"
-#ifdef HAS_UNISTD
+#include "caml/unixsupport.h"
+#ifndef _WIN32
 #include <unistd.h>
 #endif
 
-CAMLprim value unix_setsid(value unit)
+CAMLprim value caml_unix_setsid(value unit)
 {
 #ifdef HAS_SETSID
-  return Val_int(setsid());
+  pid_t pid = setsid();
+  if (pid == (pid_t)(-1)) caml_uerror("setsid", Nothing);
+  return Val_long(pid);
 #else
   caml_invalid_argument("setsid not implemented");
   return Val_unit;
