@@ -14,10 +14,10 @@
 /**************************************************************************/
 
 #include <caml/mlvalues.h>
-#include "unixsupport.h"
+#include "caml/unixsupport.h"
 
-CAMLprim value unix_setuid(value uid)
+CAMLprim value caml_unix_setuid(value uid)
 {
-  if (setuid(Int_val(uid)) == -1) uerror("setuid", Nothing);
+  if (setuid(Int_val(uid)) == -1) caml_uerror("setuid", Nothing);
   return Val_unit;
 }
