@@ -13,6 +13,7 @@
 /*                                                                        */
 /**************************************************************************/
 
+#define CAML_INTERNALS
 #include <errno.h>
 #include <caml/mlvalues.h>
 #include <caml/memory.h>
@@ -152,7 +153,7 @@ CAMLexport value caml_unix_getsockopt_aux(const char * name,
     break;
   case TYPE_TIMEVAL:
     res = caml_copy_double((double) optval.tv.tv_sec
-                           + (double) optval.tv.tv_usec / 1e6);
+                           + (double) optval.tv.tv_usec / USEC_PER_SEC);
     break;
   case TYPE_UNIX_ERROR:
     if (optval.i == 0) {
@@ -174,7 +175,6 @@ CAMLexport value caml_unix_setsockopt_aux(const char * name,
 {
   union option_value optval;
   socklen_param_type optsize;
-  double f;
 
   switch (ty) {
   case TYPE_BOOL:
@@ -189,10 +189,8 @@ CAMLexport value caml_unix_setsockopt_aux(const char * name,
       optval.lg.l_linger = Int_val(Some_val(val));
     break;
   case TYPE_TIMEVAL:
-    f = Double_val(val);
+    optval.tv = caml_timeval_of_sec(Double_val(val));
     optsize = sizeof(optval.tv);
-    optval.tv.tv_sec = (int) f;
-    optval.tv.tv_usec = (int) (1e6 * (f - optval.tv.tv_sec));
     break;
   case TYPE_UNIX_ERROR:
   default:
