@@ -13,13 +13,21 @@
 (*                                                                        *)
 (**************************************************************************)
 
-open Parsetree;;
-open Format;;
+(** Raw printer for {!Parsetree}
 
-val interface : formatter -> signature_item list -> unit;;
-val implementation : formatter -> structure_item list -> unit;;
-val top_phrase : formatter -> toplevel_phrase -> unit;;
+  {b Warning:} this module is unstable and part of
+  {{!Compiler_libs}compiler-libs}.
 
+*)
+
+open Parsetree
+open Format
+
+val interface : formatter -> signature_item list -> unit
+val implementation : formatter -> structure_item list -> unit
+val top_phrase : formatter -> toplevel_phrase -> unit
+
+val pattern: int -> formatter -> pattern -> unit
 val expression: int -> formatter -> expression -> unit
 val structure: int -> formatter -> structure -> unit
 val payload: int -> formatter -> payload -> unit
