@@ -314,18 +314,57 @@ CAMLexport CAMLweakdef void caml_initialize (volatile value *fp, value val)
     Ref_table_add(&Caml_state->minor_tables->major_ref, fp);
 }
 
-CAMLexport int caml_atomic_cas_field (
-  value obj, intnat field, value oldval, value newval)
+CAMLprim value caml_atomic_load_field (value obj, value vfield)
 {
+  intnat field = Long_val(vfield);
+  if (caml_domain_alone()) {
+    return Field(obj, field);
+  } else {
+    /* See Note [MM] above */
+    atomic_thread_fence(memory_order_acquire);
+    return atomic_load(&Op_atomic_val(obj)[field]);
+  }
+}
+CAMLprim value caml_atomic_load (value ref)
+{
+  return caml_atomic_load_field(ref, Val_long(0));
+}
+
+/* stores are implemented as exchanges */
+CAMLprim value caml_atomic_exchange_field (value obj, value vfield, value v)
+{
+  value ret;
+  intnat field = Long_val(vfield);
+  if (caml_domain_alone()) {
+    ret = Field(obj, field);
+    Field(obj, field) = v;
+  } else {
+    /* See Note [MM] above */
+    atomic_thread_fence(memory_order_acquire);
+    ret = atomic_exchange(&Op_atomic_val(obj)[field], v);
+    atomic_thread_fence(memory_order_release); /* generates `dmb ish` on Arm64*/
+  }
+  write_barrier(obj, field, ret, v);
+  return ret;
+}
+CAMLprim value caml_atomic_exchange (value ref, value v)
+{
+  return caml_atomic_exchange_field(ref, Val_long(0), v);
+}
+
+CAMLprim value caml_atomic_cas_field (
+  value obj, value vfield, value oldval, value newval)
+{
+  intnat field = Long_val(vfield);
   if (caml_domain_alone()) {
     /* non-atomic CAS since only this thread can access the object */
     volatile value* p = &Field(obj, field);
     if (*p == oldval) {
       *p = newval;
       write_barrier(obj, field, oldval, newval);
-      return 1;
+      return Val_true;
     } else {
-      return 0;
+      return Val_false;
     }
   } else {
     /* need a real CAS */
@@ -334,84 +373,38 @@ CAMLexport int caml_atomic_cas_field (
     atomic_thread_fence(memory_order_release); /* generates `dmb ish` on Arm64*/
     if (cas_ret) {
       write_barrier(obj, field, oldval, newval);
-      return 1;
+      return Val_true;
     } else {
-      return 0;
+      return Val_false;
     }
   }
 }
-
-
-CAMLprim value caml_atomic_load (value ref)
+CAMLprim value caml_atomic_cas (value ref, value oldval, value newval)
 {
-  if (caml_domain_alone()) {
-    return Field(ref, 0);
-  } else {
-    value v;
-    /* See Note [MM] above */
-    atomic_thread_fence(memory_order_acquire);
-    v = atomic_load(Op_atomic_val(ref));
-    return v;
-  }
+  return caml_atomic_cas_field(ref, Val_long(0), oldval, newval);
 }
 
-/* stores are implemented as exchanges */
-CAMLprim value caml_atomic_exchange (value ref, value v)
+CAMLprim value caml_atomic_fetch_add_field (value obj, value vfield, value incr)
 {
+  intnat field = Long_val(vfield);
   value ret;
   if (caml_domain_alone()) {
-    ret = Field(ref, 0);
-    Field(ref, 0) = v;
-  } else {
-    /* See Note [MM] above */
-    atomic_thread_fence(memory_order_acquire);
-    ret = atomic_exchange(Op_atomic_val(ref), v);
-    atomic_thread_fence(memory_order_release); /* generates `dmb ish` on Arm64*/
-  }
-  write_barrier(ref, 0, ret, v);
-  return ret;
-}
-
-CAMLprim value caml_atomic_cas (value ref, value oldv, value newv)
-{
-  if (caml_domain_alone()) {
-    value* p = Op_val(ref);
-    if (*p == oldv) {
-      *p = newv;
-      write_barrier(ref, 0, oldv, newv);
-      return Val_int(1);
-    } else {
-      return Val_int(0);
-    }
-  } else {
-    atomic_value* p = &Op_atomic_val(ref)[0];
-    int cas_ret = atomic_compare_exchange_strong(p, &oldv, newv);
-    atomic_thread_fence(memory_order_release); /* generates `dmb ish` on Arm64*/
-    if (cas_ret) {
-      write_barrier(ref, 0, oldv, newv);
-      return Val_int(1);
-    } else {
-      return Val_int(0);
-    }
-  }
-}
-
-CAMLprim value caml_atomic_fetch_add (value ref, value incr)
-{
-  value ret;
-  if (caml_domain_alone()) {
-    value* p = Op_val(ref);
-    CAMLassert(Is_long(*p));
+    value* p = &Op_val(obj)[field];
     ret = *p;
+    CAMLassert(Is_long(ret));
     *p = Val_long(Long_val(ret) + Long_val(incr));
     /* no write barrier needed, integer write */
   } else {
-    atomic_value *p = &Op_atomic_val(ref)[0];
-    ret = atomic_fetch_add(p, 2*Long_val(incr));
+    atomic_value *p = &Op_atomic_val(obj)[field];
+    ret = atomic_fetch_add(p, 2 * Long_val(incr));
     atomic_thread_fence(memory_order_release); /* generates `dmb ish` on Arm64*/
   }
   return ret;
 }
+CAMLprim value caml_atomic_fetch_add (value ref, value incr)
+{
+  return caml_atomic_fetch_add_field(ref, Val_long(0), incr);
+}
 
 CAMLexport void caml_set_fields (value obj, value v)
 {
@@ -436,8 +429,8 @@ Caml_inline value alloc_shr(mlsize_t wosize, tag_t tag, reserved_t reserved,
       return (value)NULL;
   }
 
-  dom_st->allocated_words += Whsize_wosize(wosize);
-  dom_st->allocated_words_direct += Whsize_wosize(wosize);
+  caml_update_major_allocated_words(
+    dom_st, Whsize_wosize(wosize), 1 /* direct */);
   if (dom_st->allocated_words_direct > dom_st->minor_heap_wsz / 5) {
     CAML_EV_COUNTER (EV_C_REQUEST_MAJOR_ALLOC_SHR, 1);
     caml_request_major_slice(1);
@@ -729,6 +722,17 @@ CAMLexport caml_stat_string caml_stat_strdup(const char *s)
   return result;
 }
 
+CAMLexport caml_stat_string caml_stat_memdup(const char *s, asize_t size,
+                                             asize_t *out_size)
+{
+  CAMLassert(size > 0);
+  caml_stat_block result = caml_stat_alloc(size);
+  memcpy(result, s, size);
+  if (out_size != NULL)
+    *out_size = size;
+  return result;
+}
+
 #ifdef _WIN32
 
 CAMLexport wchar_t * caml_stat_wcsdup_noexc(const wchar_t *s)
