@@ -28,11 +28,32 @@
 #include "caml/shared_heap.h"
 #include "caml/signals.h"
 #include "caml/weak.h"
+#include "caml/platform.h"
 
-value caml_dummy[] =
+static value weak_dummy[] =
   {(value)Make_header(0,Abstract_tag, NOT_MARKABLE),
+   Val_unit,
+   (value)Make_header(0,Abstract_tag, NOT_MARKABLE),
    Val_unit};
-value caml_ephe_none = (value)&caml_dummy[1];
+value caml_ephe_none = (value)&weak_dummy[1];
+value caml_ephe_locked = (value)&weak_dummy[3];
+
+/* The minor GC may lock key fields of ephemerons, and other domains
+   may complete minor GC and resume the mutator before they are unlocked.
+
+   So, when accessing an ephemeron key, if the value is caml_ephe_locked
+   then you need to wait until it becomes unlocked (i.e. any other value).
+
+   NB: Keys can only be locked by the minor GC, so once an unlocked value
+   is observed it's safe to do repeated accesses to the same key without
+   checking for locks, until the next allocation point. */
+value caml_ephe_await_key(value ephe, uintnat i)
+{
+  SPIN_WAIT {
+    value v = atomic_load_acquire(Op_atomic_val(ephe) + i);
+    if (v != caml_ephe_locked) return v;
+  }
+}
 
 struct caml_ephe_info* caml_alloc_ephe_info (void)
 {
@@ -74,7 +95,7 @@ CAMLprim value caml_weak_create (value len)
    Specificity of the cleaning phase (Phase_clean):
 
    The dead keys must be removed from the ephemerons and data removed
-   when one the keys is dead. Here we call it cleaning the ephemerons.
+   when one of the keys is dead. Here we call it cleaning the ephemerons.
    A specific phase of the GC is dedicated to this, Phase_clean. This
    phase is just after the mark phase, so the white values are dead
    values. It iterates the function caml_ephe_clean through all the
@@ -107,6 +128,7 @@ static void do_check_key_clean(value e, mlsize_t offset)
 {
   value elt;
   CAMLassert (offset >= CAML_EPHE_FIRST_KEY);
+  caml_ephe_await_key(e, offset);
 
   if (caml_gc_phase != Phase_sweep_ephe) return;
 
@@ -131,7 +153,7 @@ void caml_ephe_clean (value v) {
   hd = Hd_val(v);
   size = Wosize_hd (hd);
   for (mlsize_t i = CAML_EPHE_FIRST_KEY; i < size; i++) {
-    child = Field(v, i);
+    child = Ephe_key(v, i);
   ephemeron_again:
     if (child != caml_ephe_none && Is_block(child)) {
       if (Tag_val (child) == Forward_tag) {
@@ -156,7 +178,7 @@ void caml_ephe_clean (value v) {
     }
   }
 
-  child = Field(v, CAML_EPHE_DATA_OFFSET);
+  child = Ephe_data(v);
   if (child != caml_ephe_none) {
     if (release_data) {
       Field(v, CAML_EPHE_DATA_OFFSET) = caml_ephe_none;
@@ -181,25 +203,46 @@ static void clean_field (value e, mlsize_t offset)
     do_check_key_clean(e, offset);
 }
 
-static void do_set (value e, mlsize_t offset, value v)
+Caml_inline void ephe_write_barrier (value e, mlsize_t offset, value v)
 {
-  if (Is_block(v) && Is_young(v)) {
-    value old = Field(e, offset);
-    Field(e, offset) = v;
-    if (!(Is_block(old) && Is_young(old)))
+  if (Is_block (v) && Is_young (v)){
+    value old = Field (e, offset);
+    if (!(Is_block (old) && Is_young (old))){
       add_to_ephe_ref_table (&Caml_state->minor_tables->ephe_ref,
                              e, offset);
-  } else {
-    Field(e, offset) = v;
+    }
   }
 }
 
+CAMLno_tsan /* See caml_modify in memory.c for the tsan annotations on this
+               function. */
+static void ephe_modify (value e, mlsize_t offset, value val)
+{
+  volatile value *fp = &Field(e, offset);
+
+#if defined(WITH_THREAD_SANITIZER) && defined(NATIVE_CODE)
+  __tsan_func_entry(__builtin_return_address(0));
+#endif
+
+  ephe_write_barrier(e, offset, val);
+
+  /* See Note [MM] in memory.c */
+  atomic_thread_fence(memory_order_acquire);
+
+#if defined(WITH_THREAD_SANITIZER) && defined(NATIVE_CODE)
+  __tsan_write8((void *)fp);
+  __tsan_func_exit(NULL);
+#endif
+
+  atomic_store_release(&Op_atomic_val((value)fp)[0], val);
+}
+
 static value ephe_set_field (value e, mlsize_t offset, value el)
 {
   CAMLparam2(e,el);
 
   clean_field(e, offset);
-  do_set(e, offset, el);
+  ephe_modify(e, offset, el);
   CAMLreturn(Val_unit);
 }
 
@@ -330,8 +373,15 @@ static value ephe_get_field_copy (value e, mlsize_t offset)
     }
     infix_offs = 0;
 
-    /* Don't copy immediates or custom blocks #7279 */
-    if (!Is_block(val) || Tag_val(val) == Custom_tag) {
+    /* Don't copy immediates */
+    if (!Is_block(val)) {
+      copy = val;
+      goto some;
+    }
+
+    /* Don't copy, but do darken, custom blocks #7279 */
+    if (Tag_val(val) == Custom_tag) {
+      caml_darken (Caml_state, val, 0);
       copy = val;
       goto some;
     }
@@ -420,11 +470,10 @@ CAMLprim value caml_ephe_check_data (value e)
   return ephe_check_field (e, CAML_EPHE_DATA_OFFSET);
 }
 
-static value ephe_blit_field (value es, mlsize_t offset_s,
-                              value ed, mlsize_t offset_d, mlsize_t length)
+static value ephe_blit_keys (value es, mlsize_t offset_s,
+                             value ed, mlsize_t offset_d, mlsize_t length)
 {
   CAMLparam2(es,ed);
-  CAMLlocal1(ar);
 
   if (length == 0) CAMLreturn(Val_unit);
 
@@ -436,11 +485,13 @@ static value ephe_blit_field (value es, mlsize_t offset_s,
 
   if (offset_d < offset_s) {
     for (long i = 0; i < length; i++) {
-      do_set(ed, offset_d + i, Field(es, (offset_s + i)));
+      caml_ephe_await_key(ed, offset_d + i);
+      ephe_modify(ed, offset_d + i, Ephe_key(es, offset_s + i));
     }
   } else {
     for (long i = length - 1; i >= 0; i--) {
-      do_set(ed, offset_d + i, Field(es, (offset_s + i)));
+      caml_ephe_await_key(ed, offset_d + i);
+      ephe_modify(ed, offset_d + i, Ephe_key(es, offset_s + i));
     }
   }
   CAMLreturn(Val_unit);
@@ -459,13 +510,20 @@ CAMLprim value caml_ephe_blit_key (value es, value ofs,
   if (offset_d < CAML_EPHE_FIRST_KEY || offset_d + length > Wosize_val (ed)){
     caml_invalid_argument ("Weak.blit");
   }
-  return ephe_blit_field (es, offset_s, ed, offset_d, length);
+  return ephe_blit_keys (es, offset_s, ed, offset_d, length);
 }
 
 CAMLprim value caml_ephe_blit_data (value es, value ed)
 {
-  ephe_blit_field (es, CAML_EPHE_DATA_OFFSET, ed, CAML_EPHE_DATA_OFFSET, 1);
-  caml_darken(0, Field(ed, CAML_EPHE_DATA_OFFSET), 0);
+  /* We clean the source and destination ephemerons before performing the blit.
+   * This guarantees that none of the keys and the data fields being accessed
+   * during a blit operation is unmarked during [Phase_sweep]. */
+  caml_ephe_clean(es);
+  caml_ephe_clean(ed);
+
+  value v = Ephe_data(es);
+  ephe_modify(ed, CAML_EPHE_DATA_OFFSET, v);
+  caml_darken(Caml_state, v, 0);
   /* [ed] may be in [Caml_state->ephe_info->live] list. The data value may be
      unmarked. The ephemerons on the live list are not scanned during ephemeron
      marking. Hence, unconditionally darken the data value. */
